<?php

/**
 * @file
 * Contains \Drupal\yamlform\Controller\YamlFormResultsExportController.
 */

namespace Drupal\yamlform\Controller;

use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\yamlform\Entity\YamlFormSubmission;
use Drupal\yamlform\Utility\YamlFormEntityReferenceHelper;
use Drupal\yamlform\Utility\YamlFormOptionsHelper;
use Drupal\yamlform\YamlFormInterface;
use Drupal\yamlform\YamlFormSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Controller routines for YAML form submission export.
 */
class YamlFormResultsExportController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Default options for exporting a CSV.
   *
   * @var array
   */
  static public $defaultExportOptions = [
    'delimiter' => ',',
    'header_keys' => 'label',
    'options_format' => 'compact',
    'options_item_format' => 'label',
    'entity_reference_format' => 'link',
    'excluded_columns' => [
      'uuid' => 'uuid',
      'token' => 'token',
      'changed' => 'changed',
      'data' => 'data',
      'yamlform_id' => 'yamlform_id',
    ],
    'range_type' => 'all',
    'range_latest' => 0,
    'range_start' => '',
    'range_end' => '',
    'download' => TRUE,
  ];

  /**
   * The query factory to create entity queries.
   *
   * @var \Drupal\Core\Entity\Query\QueryFactory
   */
  public $queryFactory;

  /**
   * Constructs a YamlFormResultsExportController object.
   *
   * @param \Drupal\Core\Entity\Query\QueryFactory $query_factory
   *   The query object that can query the given entity type.
   */
  public function __construct(QueryFactory $query_factory) {
    $this->queryFactory = $query_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.query')
    );
  }

  /**
   * Returns YAML form submission as a CSV.
   */
  public function index(Request $request, YamlFormInterface $yamlform) {
    $query = $request->query->all();
    unset($query['destination']);
    if ($query) {
      if (!empty($query['excluded_columns']) && is_string($query['excluded_columns'])) {
        $excluded_columns = explode(',', $query['excluded_columns']);
        $query['excluded_columns'] = array_combine($excluded_columns, $excluded_columns);
      }
      return $this->download($yamlform, $query + self::$defaultExportOptions);
    }
    else {
      return $this->formBuilder()->getForm('Drupal\yamlform\Form\YamlFormResultsExportForm', $yamlform);
    }
  }

  /**
   * Download YAML form submission as a CSV.
   *
   * From: http://obtao.com/blog/2013/12/export-data-to-a-csv-file-with-symfony/
   *
   * @param \Drupal\yamlform\YamlFormInterface $yamlform
   *   A YAML form.
   * @param array $export_options
   *   An associative array of export options generated via the
   *   Drupal\yamlform\Form\YamlFormResultsExportForm.
   *
   * @return \Symfony\Component\HttpFoundation\StreamedResponse
   *   A streamed response that generates a CSV.
   */
  protected function download(YamlFormInterface $yamlform, array $export_options) {
    $entity_manager = $this->entityManager();
    $response = new StreamedResponse(function () use ($yamlform, $export_options, $entity_manager) {
      /** @var \Drupal\yamlform\YamlFormSubmissionStorage $submission_storage */
      $submission_storage = $this->entityManager()->getStorage('yamlform_submission');

      /** @var \Drupal\yamlform\YamlFormSubmissionViewBuilder $view_builder */
      $view_builder = $this->entityManager()->getViewBuilder('yamlform_submission');

      $query = $this->queryFactory->get('yamlform_submission')
        ->condition('yamlform_id', $yamlform->id());

      // Filter by sid or date range.
      switch ($export_options['range_type']) {
        case 'sid':
          if ($export_options['range_start']) {
            $query->condition('sid', $export_options['range_start'], '>=');
          }
          if ($export_options['range_end']) {
            $query->condition('sid', $export_options['range_end'], '<=');
          }
          break;

        case 'date':
          if ($export_options['range_start']) {
            $query->condition('created', strtotime($export_options['range_start']), '>=');
          }
          if ($export_options['range_end']) {
            $query->condition('created', strtotime($export_options['range_end']), '<=');
          }
          break;
      }

      if ($export_options['range_type'] == 'latest' && $export_options['range_latest']) {
        $query->range(0, $export_options['range_latest']);
        $query->sort('sid', 'DESC');
      }
      else {
        $query->sort('sid');
      }

      $entity_ids = $query->execute();
      if (empty($entity_ids)) {
        return;
      }

      // TODO: (PERFORMANCE) Consider preloading all entity references to improve performance.
      $yamlform_submissions = YamlFormSubmission::loadMultiple($entity_ids);

      $field_definitions = $submission_storage->getFieldDefinitions();
      $field_definitions = array_diff_key($field_definitions, $export_options['excluded_columns']);
      // Add custom entity reference field definitions which rely on the
      // entity type and entity id.
      if ($export_options['entity_reference_format'] == 'link' && isset($field_definitions['entity_type']) && isset($field_definitions['entity_id'])) {
        $field_definitions['entity_title'] = [
          'name' => 'entity_title',
          'title' => t('Submitted to: Entity title'),
          'type' => 'entity_title',
        ];
        $field_definitions['entity_url'] = [
          'name' => 'entity_url',
          'title' => t('Submitted to: Entity URL'),
          'type' => 'entity_url',
        ];
      }

      $input_columns = $yamlform->getInputColumns();
      $input_columns = array_diff_key($input_columns, $export_options['excluded_columns']);

      // Build field definition headers.
      $header = [];
      foreach ($field_definitions as $field_name => $field_definition) {
        if ($field_definition['type'] == 'entity_reference') {
          self::formatHeaderEntity($header, $field_definition['title'], $field_definition['name'], $export_options);
        }
        else {
          $header[] = ($export_options['header_keys'] == 'label') ? $field_definition['title'] : $field_definition['name'];
        }
      }

      // Build input columns headers.
      foreach ($input_columns as $input_column) {
        if ($export_options['options_format'] == 'separate' && isset($input_column['element']['#options'])) {
          $element_options = $input_column['element']['#options'];
          foreach ($element_options as $option_value => $option_text) {
            $header[] = ($export_options['options_item_format'] == 'key') ? $option_value : $option_text;
          }
        }
        elseif ($input_column['element']['#type'] == 'entity_autocomplete' && empty($input_column['element']['#tags'])) {
          self::formatHeaderEntity($header, $input_column['title'], $input_column['name'], $export_options);
        }
        else {
          $header[] = ($export_options['header_keys'] == 'label') ? $input_column['title'] : $input_column['name'];
        }
      }

      $handle = fopen('php://output', 'r+');

      // Build records.
      fputcsv($handle, $header, $export_options['delimiter']);
      foreach ($yamlform_submissions as $yamlform_submission) {
        $record = [];

        // Build record field definition columns.
        foreach ($field_definitions as $field_name => $field_definition) {
          static::formatRecordFieldDefinitionValue($record, $yamlform_submission, $field_definition, $export_options);
        }

        // Build record input columns.
        $data = $yamlform_submission->getData();
        foreach ($input_columns as $column_name => $input_column) {
          $value = (isset($data[$column_name])) ? $data[$column_name] : '';

          // Handle select, radios and checkbox options.
          if (isset($input_column['element']['#options'])) {
            YamlFormResultsExportController::formatRecordOptions($record, $value, $input_column, $export_options);
          }
          elseif ($input_column['element']['#type'] == 'entity_autocomplete') {
            YamlFormResultsExportController::formatRecordEntityAutocomplete($record, $value, $input_column, $export_options);
          }
          else {
            $value = $view_builder->formatElementValueAsText($input_column['element'], $value);
            $record[] = $value;
          }

        }

        fputcsv($handle, $record, $export_options['delimiter']);
      }

      fclose($handle);
      return;
    });

    if ($export_options['download']) {
      $response->headers->set('Content-Type', 'application/force-download');
      $response->headers->set('Content-Disposition', 'attachment; filename="' . $yamlform->id() . '.csv"');
    }
    else {
      $response->headers->set('Content-Type', 'text/plain; charset=utf-8');
    }

    return $response;
  }

  /**
   * Get the field definition value from a YAML form submission entity.
   *
   * @param array $record
   *   The record to be added to the CSV export.
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   * @param array $field_definition
   *   The field definition for the value.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordFieldDefinitionValue(array &$record, YamlFormSubmissionInterface $yamlform_submission, array $field_definition, array $export_options) {
    $field_name = $field_definition['name'];
    $field_type = $field_definition['type'];
    switch ($field_type) {
      case 'created':
      case 'changed':
        $record[] = date('c', $yamlform_submission->get($field_name)->value);
        break;

      case 'entity_reference':
        $target_type = $field_definition['target_type'];
        $target_id = $yamlform_submission->get($field_name)->target_id;
        $default_value = "$target_type:$target_id";
        self::formatRecordEntity($record, $target_type, $target_id, $default_value, $export_options);
        break;

      case 'entity_url':
      case 'entity_title':
        $entity = entity_load($yamlform_submission->entity_type->value, $yamlform_submission->entity_id->value);
        if ($entity) {
          $record[] = ($field_type == 'entity_url') ? $entity->toUrl()->setOption('absolute', TRUE)->toString() : $entity->label();
        }
        else {
          $record[] = '';
        }
        break;

      default:
        $record[] = $yamlform_submission->get($field_name)->value;
        break;

    }
  }

  /**
   * Format exported submission's record options.
   *
   * @param array $record
   *   Exported submission's record.
   * @param mixed $value
   *   The submitted value.
   * @param array $column
   *   The column/element for the submitted value.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordOptions(array &$record, $value, array $column, array $export_options) {
    $element_options = $column['element']['#options'];
    if ($export_options['options_format'] == 'separate') {
      // Separate multiple values (ie options).
      foreach ($element_options as $option_value => $option_text) {
        if (is_array($value) && isset($value[$option_value])) {
          $record[] = 'X';
        }
        elseif ($value == $option_value) {
          $record[] = 'X';
        }
        else {
          $record[] = '';
        }
      }
    }
    else {
      // Handle multiple values with options.
      if (is_array($value)) {
        if ($export_options['options_item_format'] == 'label') {
          $value = YamlFormOptionsHelper::getOptionsText($value, $element_options);
        }
        $record[] = implode(',', $value);
      }
      // Handle single values with options.
      elseif ($export_options['options_item_format'] == 'label') {
        $record[] = YamlFormOptionsHelper::getOptionText($value, $element_options);
      }
    }
  }

  /**
   * Format exported submission's record entity (reference) autocomplete.
   *
   * @param array $record
   *   Exported submission's record.
   * @param mixed $value
   *   The submitted value.
   * @param array $column
   *   The column/element for the submitted value.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordEntityAutocomplete(&$record, $value, $column, $export_options) {
    $target_type = $column['element']['#target_type'];
    if (!empty($column['element']['#tags'])) {
      $record[] = YamlFormEntityReferenceHelper::formatTargetEntities($target_type, $value, FALSE);
    }
    else {
      $target_id = $value;
      $default_value = "$target_type:$target_id";
      self::formatRecordEntity($record, $target_type, $target_id, $default_value, $export_options);
    }
  }

  /**
   * Format entity header.
   *
   * @param array $header
   *   Export header.
   * @param string $title
   *   Column title.
   * @param string $name
   *   Column name.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatHeaderEntity(array &$header, $title, $name, array $export_options) {
    if ($export_options['entity_reference_format'] == 'link') {
      if ($export_options['header_keys'] == 'label') {
        $header[] = $title . ' ID';
        $header[] = $title . ' Title';
        $header[] = $title . ' URL';
      }
      else {
        $header[] = $name . '_id';
        $header[] = $name . '_title';
        $header[] = $name . '_url';
      }
    }
    else {
      $header[] = ($export_options['header_keys'] == 'label') ? $title : $name;
    }
  }

  /**
   * Format entity as a record.
   *
   * @param array $record
   *   Exported submission's record.
   * @param string $entity_type
   *   The entity type.
   * @param string|int $entity_id
   *   The entity value.
   * @param string|int $default_value
   *   Default value to be displayed if entity is not available.
   * @param array $export_options
   *   An associative array of export options.
   */
  public static function formatRecordEntity(array &$record, $entity_type, $entity_id, $default_value, array $export_options) {
    if ($export_options['entity_reference_format'] == 'link') {
      if ($entity = entity_load($entity_type, $entity_id)) {
        $record[] = $entity->id();
        $record[] = $entity->label();
        $record[] = $entity->toUrl('canonical', ['absolute' => TRUE])->toString();
      }
      else {
        $record[] = $default_value;
        $record[] = '';
        $record[] = '';
      }
    }
    else {
      $record[] = $default_value;
    }
  }

}
