<?php

/**
 * @file
 * Contains \Drupal\yamlform\Controller\YamlFormController.
 */

namespace Drupal\yamlform\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\yamlform\Element\YamlFormCodeMirror;
use Drupal\yamlform\YamlFormSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides route responses for YAML form submissions.
 */
class YamlFormSubmissionController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * A view builder instance.
   *
   * @var \Drupal\Core\Entity\EntityViewBuilderInterface
   */
  protected $viewBuilder;

  /**
   * Constructs a YamlFormSubmissionController object.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   */
  public function __construct(EntityManagerInterface $entity_manager) {
    $this->viewBuilder = $entity_manager->getViewBuilder('yamlform_submission');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.manager')
    );
  }

  /**
   * Returns a raw data for a YAML form submission.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   A render array representing the raw data as YAML of a
   *   YAML form submission.
   */
  public function data(Request $request, YamlFormSubmissionInterface $yamlform_submission) {
    $build = [];

    $build['description'] = [
      '#markup' => $this->t("Below is the YAML form submission's fields and submitted data."),
    ];

    $yamlform_submission_view = $this->viewBuilder->view($yamlform_submission, 'yaml');
    $elements = $this->viewBuilder->build($yamlform_submission_view);
    $build['submission'] = [
      '#prefix' => '<pre class="yamlform-codemirror-runmode">',
      '#suffix' => '</pre>',
    ] + $elements;

    $build = YamlFormCodeMirror::attachLibrary($build);
    return $build;
  }

  /**
   * Returns a plain text version for a YAML form submission.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   A render array representing the plain text version of a
   *   YAML form submission.
   */
  public function text(Request $request, YamlFormSubmissionInterface $yamlform_submission) {
    $build = [];

    $build['description'] = [
      '#markup' => $this->t("Below is the YAML form submission's submitted data as plain text and could be included in email notifications."),
    ];

    $yamlform_submission_view = $this->viewBuilder->view($yamlform_submission, 'text');
    $elements = $this->viewBuilder->build($yamlform_submission_view);
    $build['submission'] = [
      '#markup' => htmlentities(\Drupal::service('renderer')->render($elements)),
      '#prefix' => '<pre class="yamlform-codemirror-runmode">',
      '#suffix' => '</pre>',
    ];

    return $build;
  }

  /**
   * Route title callback.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   The YAML form submission.
   *
   * @return array
   *   The YAML form submission as a render array.
   */
  public function title(YamlFormSubmissionInterface $yamlform_submission) {
    return $this->t(
      '@title: Submission #@id', ['@title' => $yamlform_submission->getYamlForm()->label(), '@id' => $yamlform_submission->id()]
    );
  }

}
