<?php

/**
 * @file
 * Contains \Drupal\yamlform\Controller\YamlFormTestController.
 */

namespace Drupal\yamlform\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\yamlform\YamlFormInterface;
use Drupal\Core\Utility\Token;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Provides route responses for YAML form testing.
 */
class YamlFormTestController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The token handler.
   *
   * @var \Drupal\Core\Utility\Token $token
   */
  protected $token;

  /**
   * An associative array containing test data for element types, keyed by #type.
   *
   * @var array
   */
  protected $types;

  /**
   * An associative array containing test data for element name, keyed by name.
   *
   * @var array
   */
  protected $name;

  /**
   * Constructs a new YamlFormTestController object.
   *
   * @param \Drupal\Core\Utility\Token $token
   *   The token handler.
   */
  public function __construct(Token $token) {
    $this->token = $token;
    $this->types = Yaml::parse($this->config('yamlform.settings')->get('test.types') ?: '');
    $this->names = Yaml::parse($this->config('yamlform.settings')->get('test.names') ?: '');

    // DEBUG: Load test data from config/install.
    // $settings = Yaml::parse(file_get_contents(drupal_get_path('module', 'yamlform') . '/config/install/yamlform.settings.yml'));
    // $this->types = Yaml::parse($settings['test']['types']);
    // $this->names = Yaml::parse($settings['test']['names']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('token')
    );
  }

  /**
   * Returns a form to add a new test submission to a YAML form.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\yamlform\YamlFormInterface $yamlform
   *   The YAML form this submission will be added to.
   *
   * @return array
   *   The YAML form submission form.
   */
  public function testForm(Request $request, YamlFormInterface $yamlform) {
    if ($request->query->get('yamlform_id') == $yamlform->id()) {
      return $yamlform->getSubmissionForm();
    }

    drupal_set_message($this->t("The below form has been prepopulated with custom/random test data. When submitted, this information <b>will still be saved</b> and/or <b>sent to designated recipent's</b>."), 'warning');
    $inputs = $yamlform->getFlattenedInputs();

    $token_data = ['yamlform' => $yamlform];
    $data = [];
    foreach ($inputs as $key => $input) {
      if (!empty($input['#type'])) {
        $values = $this->getTestValues($yamlform, $key, $input);
        if ($values === NULL) {
          continue;
        }

        // Get random test value.
        if (is_array($values)) {
          $value = $values[array_rand($values)];
        }
        else {
          $value = $values;
        }

        // Replace tokens.
        if (is_string($value)) {
          $value = $this->token->replace($value, $token_data);
        }
        elseif (is_array($value)) {
          foreach (array_keys($value) as $key) {
            if (is_string($value[$key])) {
              $value[$key] = $this->token->replace($value[$key], $token_data);
            }
          }
        }

        // Checkboxes or TableSelect and #multiple require an array as the
        // default value.
        if ((in_array($input['#type'], ['checkboxes', 'tableselect']) || !empty($input['#multiple'])) && !is_array($value)) {
          $value = [$value];
        }

        $data[$key] = $value;
      }
    }

    // DEBUG:
    return $yamlform->getSubmissionForm(['data' => $data]);
  }

  /**
   * Route title callback.
   *
   * @param \Drupal\yamlform\YamlFormInterface $yamlform
   *   The YAML form.
   *
   * @return string
   *   The YAML form label as a render array.
   */
  public function title(YamlFormInterface $yamlform) {
    return $this->t('Testing %title form', ['%title' => $yamlform->label()]);
  }

  /**
   * Get test value for a YAML form element
   *
   * @param \Drupal\yamlform\YamlFormInterface $yamlform
   *   A YAML form.
   * @param string $name
   *   The name of the element.
   * @param array $element
   *   The FAPI element.
   *
   * @return array|int|null
   *   An array containing multiple values or a single value.
   */
  protected function getTestValues(YamlFormInterface $yamlform, $name, array $element) {
    // Get test value from the actual element.
    if (isset($element['#test'])) {
      return $element['#test'];
    }

    // Never populate hidden and value elements.
    if (in_array($element['#type'], ['hidden', 'value'])) {
      return NULL;
    }

    // Get test values from options.
    if (isset($element['#options'])) {
      $options = $yamlform->getElementOptions($element);
      return array_keys($options);
    }

    // Get test values using #type.
    if (isset($this->types[$element['#type']])) {
      return $this->types[$element['#type']];
    }

    // Get test values using on exact name matches.
    if (isset($this->types[$name])) {
      return $this->types[$name];
    }

    // Get test values using partial name matches.
    foreach ($this->names as $key => $values) {
      if (preg_match('/(^|_)' . $key . '(_|$)/i', $name)) {
        return $values;
      }
    }

    switch ($element['#type']) {
      case 'range';
      case 'number';
        $element += ['#min' => 1, '#max' => 10];
        return rand($element['#min'], $element['#max']);
    }

    // Return default values.
    return (isset($this->names['default'])) ? $this->names['default'] : NULL;
  }

}
