<?php

/**
 * @file
 * Contains \Drupal\yamlform\Element\YamlFormCodeMirror.
 */

namespace Drupal\yamlform\Element;

use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Render\Element\Textarea;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a form element for input of YAML text using CodeMirror.
 *
 * @FormElement("yamlform_codemirror")
 */
class YamlFormCodeMirror extends Textarea {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#cols' => 60,
      '#rows' => 5,
      '#resizable' => 'vertical',
      '#process' => [
        [$class, 'processYamlFormCodeMirror'],
        [$class, 'processAjaxForm'],
        [$class, 'processGroup'],
      ],
      '#pre_render' => [
        [$class, 'preRenderYamlFormCodeMirror'],
        [$class, 'preRenderGroup'],
      ],
      '#element_validate' => [
        [$class, 'validateYamlFormCodeMirror'],
      ],
      '#theme' => 'textarea',
      '#theme_wrappers' => ['form_element'],
    ];
  }

  /**
   * Processes a YAML form element.
   */
  public static function processYamlFormCodeMirror(&$element, FormStateInterface $form_state, &$complete_form) {
    return self::attachLibrary($element);
  }

  /**
   * Form element validation handler for #type 'yaml_code'.
   */
  public static function validateYamlFormCodeMirror(&$element, FormStateInterface $form_state, &$complete_form) {
    try {
      $data = Yaml::decode($element['#value']);
      if (!is_array($data)) {
        throw new \Exception(t('YAML must contain an associative array of inputs.'));
      }
    }
    catch (\Exception $exception) {
      $form_state->setError($element, t('%title is not valid. @message', ['%title' => $element['#title'], '@message' => $exception->getMessage()]));
    }
  }

  /**
   * Prepares a #type 'yamlform_codemirror' render element for theme_input().
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   *   Properties used: #title, #value, #description, #size, #maxlength,
   *   #placeholder, #required, #attributes.
   *
   * @return array
   *   The $element with prepared variables ready for theme_input().
   */
  public static function preRenderYamlFormCodeMirror($element) {
    static::setAttributes($element, ['yamlform-codemirror']);
    return $element;
  }

  /**
   * Attach YAML form CodeMirror library to render element.
   *
   * @param array $element
   *   A render array element.
   *
   * @return array
   *   A render array with YAML form CodeMirror library attached.
   */
  public static function attachLibrary(array &$element) {
    $element['#attached']['library'][] = 'yamlform/yamlform.codemirror';
    $element['#attached']['library'][] = 'yamlform/' . (file_exists(DRUPAL_ROOT . '/libraries/codemirror') ? 'libraries' : 'cdn') . '.codemirror';
    return $element;
  }

}
