<?php

/**
 * @file
 * Contains Drupal\yamlform\Entity\YamlForm.
 */

namespace Drupal\yamlform\Entity;

use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Config\Entity\ConfigEntityBundleBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityWithPluginCollectionInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\yamlform\YamlFormHandlerInterface;
use Drupal\yamlform\YamlFormHandlerPluginCollection;
use Drupal\yamlform\YamlFormInterface;

/**
 * Defines the YAML form entity.
 *
 * @ConfigEntityType(
 *   id = "yamlform",
 *   label = @Translation("YAML form"),
 *   handlers = {
 *     "view_builder" = "Drupal\yamlform\YamlFormEntityViewBuilder",
 *     "list_builder" = "Drupal\yamlform\YamlFormEntityListBuilder",
 *     "access" = "Drupal\yamlform\YamlFormEntityAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\yamlform\YamlFormEntityForm",
 *       "settings" = "Drupal\yamlform\YamlFormEntitySettingsForm",
 *       "handlers" = "Drupal\yamlform\YamlFormEntityHandlersForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *       "duplicate" = "Drupal\yamlform\YamlFormEntityForm",
 *     }
 *   },
 *   admin_permission = "administer yamlform",
 *   bundle_of = "yamlform_submission",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "title",
 *   },
 *   common_reference_target = TRUE,
 *   links = {
 *     "canonical" = "/yamlform/{yamlform}",
 *     "add-form" = "/yamlform/{yamlform}",
 *     "test-form" = "/yamlform/{yamlform}/test",
 *     "edit-form" = "/admin/structure/yamlform/manage/{yamlform}",
 *     "settings-form" = "/admin/structure/yamlform/manage/{yamlform}/settings",
 *     "handlers-form" = "/admin/structure/yamlform/manage/{yamlform}/handlers",
 *     "duplicate-form" = "/admin/structure/yamlform/manage/{yamlform}/duplicate",
 *     "delete-form" = "/admin/structure/yamlform/manage/{yamlform}/delete",
 *     "results-submissions" = "/admin/structure/yamlform/manage/{yamlform}/results/submissions",
 *     "results-table" = "/admin/structure/yamlform/manage/{yamlform}/results/table",
 *     "results-export" = "/admin/structure/yamlform/manage/{yamlform}/results/export",
 *     "results-clear" = "/admin/structure/yamlform/manage/{yamlform}/results/clear",
 *     "collection" = "/admin/structure/yamlform",
 *   },
 *   config_export = {
 *     "status",
 *     "id",
 *     "uuid",
 *     "title",
 *     "description",
 *     "inputs",
 *     "settings",
 *     "handlers",
 *   }
 * )
 */
class YamlForm extends ConfigEntityBundleBase implements EntityWithPluginCollectionInterface, YamlFormInterface {

  use StringTranslationTrait;

  /**
   * The YAML form ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The YAML form UUID.
   *
   * @var string
   */
  protected $uuid;

  /**
   * The YAML form status.
   *
   * @var boolean
   */
  protected $status = 1;

  /**
   * The YAML form title.
   *
   * @var string
   */
  protected $title;

  /**
   * The YAML form description.
   *
   * @var string
   */
  protected $description;

  /**
   * The YAML form settings.
   *
   * @var array
   */
  protected $settings = [];

  /**
   * The YAML form inputs.
   *
   * @var string
   */
  protected $inputs;

  /**
   * The array of YAML form handlers for this YAML form.
   *
   * @var array
   */
  protected $handlers = [];

  /**
   * Holds the collection of YAML form handlers that are used by this YAML form.
   *
   * @var \Drupal\yamlform\YamlFormHandlerPluginCollection
   */
  protected $handlersCollection;


  /**
   * The YAML form input inputs decoded.
   *
   * @var array
   */
  protected $inputsDecoded;

  /**
   * The YAML form input flattened.
   *
   * @var array
   */
  protected $flattenedInputs;

  /**
   * The YAML form input columns.
   *
   * @var array
   */
  protected $inputColumns;

  /**
   * Track if the form has a managed file (upload) element.
   *
   * @var bool
   */
  protected $hasManagedFile = FALSE;

  /**
   * {@inheritdoc}
   */
  public function isOpen() {
    return $this->status ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isClosed() {
    return !$this->isOpen();
  }

  /**
   * {@inheritdoc}
   */
  public function hasManagedFile() {
    $this->initInputs();
    return $this->hasManagedFile;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettings() {
    return $this->settings + $this->getDefaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function setSettings(array $settings) {
    $this->settings = $settings + $this->getDefaultSettings();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultSettings() {
    return [
      'form_submit_label' => '',
      'form_closed_message' => '',
      'form_prepopulate' => FALSE,
      'preview' => DRUPAL_DISABLED,
      'preview_next_button_label' => '',
      'preview_prev_button_label' => '',
      'preview_message' => '',
      'confirmation_type' => 'page',
      'confirmation_message' => '',
      'confirmation_url' => '',
      'results_disabled' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSubmissionForm(array $values = []) {
    // Set this YAML form's id.
    $values['yamlform_id'] = $this->id();

    $yamlform_submission = $this->entityManager()
      ->getStorage('yamlform_submission')
      ->create($values);

    return \Drupal::service('entity.form_builder')
      ->getForm($yamlform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function getInputs() {
    $this->initInputs();
    return $this->inputsDecoded;
  }

  /**
   * {@inheritdoc}
   */
  public function getFlattenedInputs() {
    $this->initInputs();
    return $this->flattenedInputs;
  }

  /**
   * Initialize parse YAML form inputs.
   */
  protected function initInputs() {
    if (isset($this->inputsDecoded)) {
      return;
    }
    $this->flattenedInputs = [];
    try {
      $inputs = Yaml::decode($this->inputs);
      // Since YAML supports simple values.
      $inputs = (is_array($inputs)) ? $inputs : [];
    }
    catch (\Exception $exception) {
      $link = $this->link(t('Edit'), 'edit-form');
      \Drupal::logger('yamlform')
        ->notice('%title inputs are not valid. @message', [
          '%title' => $this->label(),
          '@message' => $exception->getMessage(),
          'link' => $link,
        ]);
      $inputs = FALSE;
    }

    if ($inputs !== FALSE) {
      $this->initInputsRecursive($inputs);
      $this->invokeHandlers('alterInputs', $inputs, $this);
    }

    $this->inputsDecoded = $inputs;
  }

  /**
   * Flatten nested array of elements.
   *
   * @param array $elements
   */
  protected function initInputsRecursive(array &$elements) {
    foreach ($elements as $key => &$element) {
      if (Element::property($key) || !is_array($element)) {
        continue;
      }
      // Set key.
      $element['#key'] = $key;

      // Track managed file upload.
      if (isset($element['#type']) && $element['#type'] == 'managed_file') {
        $this->hasManagedFile = TRUE;
      }

      $this->flattenedInputs[$key] = $element;
      $this->initInputsRecursive($element);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getInputColumns() {
    if (!isset($this->inputColumns)) {
      $this->inputColumns = [];
      $inputs = $this->getFlattenedInputs();
      foreach ($inputs as $key => $element) {
        if (empty($element['#type']) || in_array($element['#type'], ['container', 'details', 'fieldset', 'item', 'label'])) {
          continue;
        }

        if (isset($element['#options'])) {
          $element['#options'] = $this->getElementOptions($element);
        }

        $this->inputColumns[$key] = [
          'name' => $key,
          'title' => (!empty($element['#title'])) ? $element['#title'] : $key,
          'element' => $element,
        ];
      }
    }
    return $this->inputColumns;
  }

  /**
   * {@inheritdoc}
   */
  public function getElementOptions(array $element) {
    if (is_array($element['#options'])) {
      return $element['#options'];
    }
    elseif (!empty($element['#options']) && is_string($element['#options']) && ($yamlform_options = YamlFormOptions::load($element['#options']))) {
      return $yamlform_options->getOptions();
    }
    else {
      $link = $this->link($this->t('Edit'), 'edit-form');
      \Drupal::logger('yamlform')->notice('%options YAML form options do not exist.', ['%form' => $this->label(), 'link' => $link]);
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function preDelete(EntityStorageInterface $storage, array $entities) {
    parent::preDelete($storage, $entities);

    // Delete all submission associated with this YAML form.
    $entity_ids = \Drupal::entityQuery('yamlform_submission')
      ->condition('yamlform_id', array_keys($entities), 'IN')
      ->sort('sid')
      ->execute();
    entity_delete_multiple('yamlform_submission', $entity_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $cache_tags = parent::getCacheTags();
    // Add YAML form to cache tags which are used by the YamlFormSubmissionForm.
    $cache_tags[] = 'yamlform:' . $this->id();
    return $cache_tags;
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);

    // Clear cached properties.
    $this->inputsDecoded = NULL;
    $this->flattenedInputs = NULL;
    $this->inputColumns = NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteYamlFormHandler(YamlFormHandlerInterface $handler) {
    $this->getHandlers()->removeInstanceId($handler->getHandlerId());
    $this->save();
    return $this;
  }


  /**
   * Returns the YAML form handler plugin manager.
   *
   * @return \Drupal\Component\Plugin\PluginManagerInterface
   *   The YAML form handler plugin manager.
   */
  protected function getYamlFormHandlerPluginManager() {
    return \Drupal::service('plugin.manager.yamlform.handler');
  }

  /**
   * {@inheritdoc}
   */
  public function getHandler($handler) {
    return $this->getHandlers()->get($handler);
  }

  /**
   * {@inheritdoc}
   */
  public function getHandlers($plugin_id = NULL) {
    if (!$this->handlersCollection) {
      $this->handlersCollection = new YamlFormHandlerPluginCollection($this->getYamlFormHandlerPluginManager(), $this->handlers);
      /** @var \Drupal\yamlform\YamlFormHandlerBase $handler */
      foreach ($this->handlersCollection as $handler) {
        // Initialize the handler and pass in the YAML form.
        $handler->init($this);
      }
      $this->handlersCollection->sort();
    }

    // Filter the handlers by plugin id.
    // This is used to limit track and enforce a handlers cardinality.
    if ($plugin_id) {
      /** @var \Drupal\yamlform\YamlFormHandlerPluginCollection $handlers */
      $handlers = clone $this->handlersCollection;
      foreach ($handlers as $instance_id => $handler) {
        if ($handler->getPluginId() != $plugin_id) {
          $handlers->removeInstanceId($instance_id);
        }
      }
      return $handlers;
    }
    else {
      return $this->handlersCollection;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginCollections() {
    return ['handlers' => $this->getHandlers()];
  }

  /**
   * {@inheritdoc}
   */
  public function addYamlFormHandler(array $configuration) {
    $this->getHandlers()->addInstanceId($configuration['handler_id'], $configuration);
    return $configuration['handler_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function invokeHandlers($method, &$data, &$context1 = NULL, &$context2 = NULL) {
    $handlers = $this->getHandlers();
    foreach ($handlers as $handler) {
      $handler->$method($data, $context1, $context2);
    }
  }

  /**
   * {@inheritdoc}
   *
   * Overriding so that URLs pointing to YAML form default to 'canonical'
   * submission form and not the back-end 'edit-form'.
   */
  public function url($rel = 'canonical', $options = []) {
    // Do not remove this override: the default value of $rel is different.
    return parent::url($rel, $options);
  }

  /**
   * {@inheritdoc}
   *
   * Overriding so that URLs pointing to YAML form default to 'canonical'
   * submission form and not the back-end 'edit-form'.
   */
  public function toUrl($rel = 'canonical', array $options = []) {
    return parent::toUrl($rel, $options);
  }

  /**
   * {@inheritdoc}
   *
   * Overriding so that URLs pointing to YAML form default to 'canonical'
   * submission form and not the back-end 'edit-form'.
   */
  public function urlInfo($rel = 'canonical', array $options = []) {
    return parent::urlInfo($rel, $options);
  }

  /**
   * {@inheritdoc}
   *
   * Overriding so that links to YAML form default to 'canonical' submission
   * form and not the back-end 'edit-form'.
   */
  public function toLink($text = NULL, $rel = 'canonical', array $options = []) {
    return parent::toLink($text, $rel, $options);
  }

  /**
   * {@inheritdoc}
   *
   * Overriding so that links to YAML form default to 'canonical' submission
   * form and not the back-end 'edit-form'.
   */
  public function link($text = NULL, $rel = 'canonical', array $options = []) {
    return parent::link($text, $rel, $options);
  }

  /**
   * {@inheritdoc}
   */
  public function isDefaultRevision() {
    return TRUE;
  }

}
