<?php
/**
 * @file
 * Contains \Drupal\yamlform\Entity\YamlFormSubmission.
 */

namespace Drupal\yamlform\Entity;

use Drupal\Component\Serialization\Yaml;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\file\Entity\File;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Drupal\yamlform\YamlFormSubmissionInterface;

/**
 * Defines the YamlFormSubmission entity.
 *
 * @ingroup yamlform
 *
 * @ContentEntityType(
 *   id = "yamlform_submission",
 *   label = @Translation("YAML form submission"),
 *   bundle_label = @Translation("YAML form"),
 *   handlers = {
 *     "storage" = "Drupal\yamlform\YamlFormSubmissionStorage",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "view_builder" = "Drupal\yamlform\YamlFormSubmissionViewBuilder",
 *     "list_builder" = "Drupal\yamlform\YamlFormSubmissionListBuilder",
 *     "access" = "Drupal\yamlform\YamlFormSubmissionAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\yamlform\YamlFormSubmissionForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *   },
 *   bundle_entity_type = "yamlform",
 *   list_cache_contexts = { "user" },
 *   base_table = "yamlform_submission",
 *   admin_permission = "administer yamlform",
 *   entity_keys = {
 *     "id" = "sid",
 *     "bundle" = "yamlform_id",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/yamlform/results/manage/{yamlform_submission}",
 *     "edit-form" = "/admin/structure/yamlform/results/manage/{yamlform_submission}/edit",
 *     "delete-form" = "/admin/structure/yamlform/results/manage/{yamlform_submission}/delete",
 *     "collection" = "/admin/structure/yamlform/results/manage/list"
 *   },
 *   permission_granularity = "bundle"
 * )
 */
class YamlFormSubmission extends ContentEntityBase implements YamlFormSubmissionInterface {

  use EntityChangedTrait;

  /**
   * The decoded data.
   *
   * @var array
   */
  protected $decodedData = [];

  /**
   * Reference to original data loaded before any updates.
   *
   * @var array
   */
  protected $originalData = [];

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $weight = -20;
    $fields['sid'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Submission ID'))
      ->setDescription(t('The ID of the YAML form submission entity.'))
      ->setReadOnly(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('Submission UUID'))
      ->setDescription(t('The UUID of the YAML form submission entity.'))
      ->setReadOnly(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['token'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Token'))
      ->setDescription(t('A secure token used to look up a submission.'))
      ->setSetting('max_length', 255);

    $fields['uri'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Submission URI'))
      ->setDescription(t('The URI the user submitted the form.'))
      ->setSetting('max_length', 2000)
      ->setReadOnly(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the YAML form submission was created.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the YAML form submission was last edited.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['remote_addr'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Remote IP address'))
      ->setDescription(t('The IP address of the user that submitted the form.'))
      ->setSetting('max_length', 128)
      ->setReadOnly(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Submitted by'))
      ->setDescription(t('The submitter.'))
      ->setSetting('target_type', 'user')
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'entity_reference',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['yamlform_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Form'))
      ->setDescription(t('The associated yamlform.'))
      ->setSetting('target_type', 'yamlform')
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'entity_reference',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['entity_type'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Submitted to: Entity type'))
      ->setDescription(t('The entity type to which this submission was submitted from.'))
      ->setSetting('is_ascii', TRUE)
      ->setSetting('max_length', EntityTypeInterface::ID_MAX_LENGTH)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    // Can't use entity reference without a target type because it defaults to
    // an integer which limits reference to only content entities (and not
    // config entities like Views, Panels, etc...).
    // See: \Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem::propertyDefinitions.
    $fields['entity_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Submitted to: Entity ID'))
      ->setDescription(t('The ID of the entity of which this YAML form submission was submitted from.'))
      ->setSetting('max_length', 255)
      ->setReadOnly(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'default',
        'weight' => $weight++,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['data'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Data (YAML)'))
      ->setDescription(t('The data submitted to the YAML form serialized as YAML.'))
      ->setDisplayOptions('view', [
        'type' => 'default',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getData($key = NULL) {
    // Using empty() instead of isset() to ensure new submission data is updated.
    if (empty($this->decodedData)) {
      $this->decodedData = $this->data->value ? Yaml::decode($this->data->value) : [];
    }

    if ($key !== NULL) {
      return (isset($this->decodedData[$key])) ? $this->decodedData[$key] : NULL;
    }
    else {
      return $this->decodedData;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setData(array $data) {
    $this->data->value = Yaml::encode($data);
    $this->decodedData = $data;
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginalData($key = NULL) {
    if ($key !== NULL) {
      return (isset($this->originalData[$key])) ? $this->originalData[$key] : NULL;
    }
    else {
      return $this->originalData;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getToken() {
    return $this->token->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getYamlForm() {
    return $this->yamlform_id->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceEntity() {
    if ($this->entity_type->value && $this->entity_id->value) {
      return entity_load($this->entity_type->value, $this->entity_id->value);
    }
    else {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceUrl() {
    $uri = $this->uri->value;
    if ($url = \Drupal::pathValidator()->getUrlIfValid($uri)) {
      return $url->setOption('absolute', TRUE);
    }
    elseif ($entity = $this->getSourceEntity()) {
      return $entity->toUrl()->setOption('absolute', TRUE);
    }
    else {
      return $this->getYamlForm()->toUrl()->setOption('absolute', TRUE);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function invokeYamlFormHandlers($method, &$context1 = NULL, &$context2 = NULL) {
    $this->getYamlForm()->invokeHandlers($method, $this, $context1, $context2);
  }

  /**
   * {@inheritdoc}
   */
  public function invokeYamlFormElement($method, array $element) {
    /** @var \Drupal\yamlform\YamlFormElementManager $element_manager */
    $element_manager = \Drupal::service('plugin.manager.yamlform.element');
    $element_manager->invokeMethod($method, $element, $this);
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    $user = $this->get('uid')->entity;
    if (!$user || $user->isAnonymous()) {
      $user = User::getAnonymousUser();
    }
    return $user;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage, array &$values) {
    if (empty($values['yamlform_id'])) {
      throw new \Exception('YAML form id (yamlform_id) is required to create a YAML form submission.');
    }

    // Set default uri and remote_addr.
    $current_request = \Drupal::requestStack()->getCurrentRequest();
    $values += [
      'uri' => preg_replace('#^' . base_path() . '#', '/', $current_request->getRequestUri()),
      'remote_addr' => $current_request->getClientIp(),
    ];

    // Get default uid.
    $current_user = \Drupal::currentUser();
    $values += [
      'uid' => $current_user->id(),
    ];

    // Hard code the token.
    $values['token'] = Crypt::randomBytesBase64(55);

    // Serialize the data array.
    if (isset($values['data']) && is_array($values['data'])) {
      $values['data'] = Yaml::encode($values['data']);
    }

    // Get request's entity parameter.
    $entity_types = \Drupal::entityManager()->getEntityTypeLabels();
    foreach ($entity_types as $entity_type => $entity_label) {
      $entity = \Drupal::routeMatch()->getParameter($entity_type);
      if ($entity instanceof EntityInterface) {
        $values += [
          'entity_type' => $entity->getEntityTypeId(),
          'entity_id' => $entity->id(),
        ];
        break;
      }
    }

    YamlForm::load($values['yamlform_id'])->invokeHandlers(__FUNCTION__, $values);
  }

  /**
   * {@inheritdoc}
   */
  public function postCreate(EntityStorageInterface $storage) {
    parent::postCreate($storage);
    $this->invokeYamlFormHandlers(__FUNCTION__);
  }

  /**
   * {@inheritdoc}
   */
  public static function postLoad(EntityStorageInterface $storage, array &$entities) {
    foreach ($entities as $entity) {
      $entity->originalData = $entity->getData();
      $entity->invokeYamlFormHandlers(__FUNCTION__);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    foreach ($entities as $entity) {
      $inputs = $entity->getYamlForm()->getFlattenedInputs();
      foreach ($inputs as $key => $element) {
        $entity->invokeYamlFormElement(__FUNCTION__, $element);
      }
      $entity->invokeYamlFormHandlers(__FUNCTION__);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function preDelete(EntityStorageInterface $storage, array $entities) {
    parent::preDelete($storage, $entities);
    foreach ($entities as $entity) {
      $entity->invokeYamlFormHandlers(__FUNCTION__);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
    $this->invokeYamlFormHandlers(__FUNCTION__);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);
    $this->invokeYamlFormHandlers(__FUNCTION__, $update);
  }

  /**
   * {@inheritdoc}
   */
  public function save() {
    $result = parent::save();

    $inputs = $this->getYamlForm()->getFlattenedInputs();
    foreach ($inputs as $key => $element) {
      $this->invokeYamlFormElement(__FUNCTION__, $element);
    }

    // TODO: Move below code to 'managed_file' plugin.
    // $this->updateManagedFileUsage();

    // Reset cached properties.
    $this->decodedData = NULL;
    $this->originalData = $this->getData();

    return $result;
  }

  /**
   * Update managed file usage after submission save.
   */
  protected function updateManagedFileUsage() {
    // Make sure the file.module is enabled.
    if (!\Drupal::moduleHandler()->moduleExists('file')) {
      return;
    }

    /** @var \Drupal\file\FileUsage\FileUsageInterface $file_usage */
    $file_usage = \Drupal::service('file.usage');

    /** @var \Drupal\yamlform\YamlFormInterface $yamlform */
    $yamlform = $this->getYamlForm();

    // Set file usage which will also make the file's status permanent.
    $original_data = $this->getOriginalData();
    $data = $this->getData();

    $inputs = $yamlform->getFlattenedInputs();
    foreach ($inputs as $key => $element) {
      if ($element['#type'] != 'managed_file') {
        continue;
      }

      // Have to use $element['#key'].
      $value = isset($data[$key]) ? $data[$key] : NULL;
      $original_value = isset($original_data[$key]) ? $original_data[$key] : NULL;

      // Check the original submission and delete the old file upload.
      if ($original_value && $original_value != $value) {
        file_delete($original_value);
      }

      $file = File::load($value);
      if (!$file) {
        continue;
      }

      $source_uri = $file->getFileUri();

      // Replace /_sid_/ token with the submission id.
      if (strpos($source_uri, '/_sid_/')) {
        $destination_uri = str_replace('/_sid_/', '/' . $this->id() . '/', $source_uri);
        $destination_directory = drupal_dirname($destination_uri);
        file_prepare_directory($destination_directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);
        $destination_uri = file_unmanaged_move($source_uri, $destination_uri);
        // Update the file's uri and save.
        $file->setFileUri($destination_uri);
        $file->save();
      }

      // Update file usage table.
      $file_usage->delete($file, 'yamlform', 'yamlform_submission', $this->id(), 0);
      $file_usage->add($file, 'yamlform', 'yamlform_submission', $this->id());
    }
  }

  /**
   * Delete all managed files associated with this submission.
   */
  protected function deleteManagedFiles() {
    // Make sure the file.module is enabled.
    if (!\Drupal::moduleHandler()->moduleExists('file')) {
      return;
    }

    /** @var \Drupal\yamlform\YamlFormInterface $yamlform */
    $yamlform = $this->getYamlForm();

    $data = $this->getData();
    $inputs = $yamlform->getFlattenedInputs();
    foreach ($inputs as $key => $input) {
      if ($input['#type'] == 'managed_file' && !empty($data[$key])) {
        file_delete($data[$key]);
      }
    }

    // Remove the empty directory.
    file_unmanaged_delete_recursive('public://yamlform//' . $yamlform->id() . '/' . $this->id());
  }

}
