<?php

/**
 * @file
 * Contains \Drupal\yamlform\Form\YamlFormAdminSettingsForm.
 */

namespace Drupal\yamlform\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Plugin\Field\FieldType\FileItem;

/**
 * Configure YAML form admin settings for this site.
 */
class YamlFormAdminSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'yamlform_admin_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['yamlform.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('yamlform.settings');
    $settings = $config->get('settings');
    $form['form'] = [
      '#type' => 'details',
      '#title' => $this->t('Form default settings'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['form']['default_form_closed_message']  = [
      '#type' => 'textarea',
      '#title' => $this->t('Closed message'),
      "#rows" => 6,
      '#required' => TRUE,
      '#default_value' => $config->get('settings.default_form_closed_message'),
    ];
    $form['form']['default_form_submit_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Submit button label'),
      '#required' => TRUE,
      '#size' => 20,
      '#default_value' => $settings['default_form_submit_label'],
    ];

    $form['preview'] = [
      '#type' => 'details',
      '#title' => $this->t('Preview default settings'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['preview']['default_preview_next_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default preview button label'),
      '#required' => TRUE,
      '#size' => 20,
      '#default_value' => $settings['default_preview_next_button_label'],
    ];
    $form['preview']['default_preview_prev_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default previous page button label'),
      '#required' => TRUE,
      '#size' => 20,
      '#default_value' => $settings['default_preview_prev_button_label'],
    ];
    $form['preview']['default_preview_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Default preview message'),
      '#required' => TRUE,
      '#default_value' => $settings['default_preview_message'],
    ];

    $form['inputs'] = [
      '#type' => 'details',
      '#title' => $this->t('Inputs default settings'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['inputs']['default_max_filesize'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Maximum upload size'),
      '#default_value' => $config->get('inputs.default_max_filesize'),
      '#description' => $this->t('Enter a value like "512" (bytes), "80 KB" (kilobytes) or "50 MB" (megabytes) in order to restrict the allowed file size. If left empty the file sizes will be limited only by PHP\'s maximum post and file upload sizes (current limit <strong>%limit</strong>).', ['%limit' => format_size(file_upload_max_size())]),
      '#element_validate' => [[get_class($this), 'validateMaxFilesize']],
      '#size' => 10,
    ];
    $form['inputs']['default_file_extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Allowed file extensions'),
      '#default_value' => $config->get('inputs.default_file_extensions'),
      '#description' => $this->t('Separate extensions with a space and do not include the leading dot.'),
      '#maxlength' => 256,
      '#element_validate' => [[get_class($this), 'validateExtensions']],
      '#required' => TRUE,
    ];

    // Mail.
    $form['mail'] = [
      '#type' => 'details',
      '#title' => $this->t('Email default settings'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['mail']['default_from_mail']  = [
      '#type' => 'email',
      '#title' => $this->t('Email from address'),
      '#default_value' => $config->get('mail.default_from_mail'),
      '#description' => $this->t('The default sender address for emailed YAML form results; often the email address of the maintainer of your forms.'),
    ];
    $form['mail']['default_from_name']  = [
      '#type' => 'textfield',
      '#title' => $this->t('Email from name'),
      '#default_value' => $config->get('mail.default_from_name'),
      '#description' => $this->t('The default sender name which is used along with the default from email address.'),
    ];
    $form['mail']['default_subject']  = [
      '#type' => 'textfield',
      '#title' => $this->t('Email subject'),
      '#default_value' => $config->get('mail.default_subject'),
      '#description' => $this->t('The default subject line of any emailed results.'),
    ];
    $form['mail']['default_body']  = [
      '#type' => 'textarea',
      '#title' => $this->t('Email body'),
      "#rows" => 6,
      '#default_value' => $config->get('mail.default_body'),
      '#description' => $this->t('The default body of any emailed results.'),
    ];
    $form['mail']['token_tree'] = [
      '#theme' => 'token_tree',
      '#token_types' => [
        'yamlform',
        'yamlform-submission',
      ],
      '#dialog' => TRUE,
    ];

    // Display.
    $form['display'] = [
      '#type' => 'details',
      '#title' => $this->t('Display settings'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['display']['submission_text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Submission plain text template'),
      "#rows" => 6,
      '#default_value' => $config->get('display.submission_text'),
    ];
    $form['display']['token_tree'] = [
      '#theme' => 'token_tree',
      '#token_types' => [
        'yamlform',
        'yamlform-submission',
      ],
      '#dialog' => TRUE,
    ];

    // Test.
    $form['test'] = [
      '#type' => 'details',
      '#title' => $this->t('Test settings'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['test']['types'] = [
      '#type' => 'yamlform_codemirror',
      '#title' => $this->t('Test data by input type'),
      '#description' => $this->t("Above test data is keyed by FAPI element #type."),
      '#default_value' => $config->get('test.types'),
    ];
    $form['test']['names'] = [
      '#type' => 'yamlform_codemirror',
      '#title' => $this->t('Test data by input name'),
      '#description' => $this->t("Above test data is keyed by full or partial inputs name. For example, 'zip' will populate fields that are named 'zip' and 'zip_code' but not 'zipcode' or 'zipline'."),
      '#default_value' => $config->get('test.names'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('yamlform.settings')
      ->set('settings', $form_state->getValue('form') + $form_state->getValue('preview'))
      ->set('inputs', $form_state->getValue('inputs'))
      ->set('mail', $form_state->getValue('mail'))
      ->set('display', $form_state->getValue('display'))
      ->set('test', $form_state->getValue('test'))
      ->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Wrapper for FileItem::validateExtensions.
   */
  public static function validateExtensions($element, FormStateInterface $form_state) {
    FileItem::validateExtensions($element, $form_state);
  }

  /**
   * Wrapper for FileItem::validateMaxFilesize.
   */
  public static function validateMaxFilesize($element, FormStateInterface $form_state) {
    FileItem::validateMaxFilesize($element, $form_state);
  }

}
