<?php

/**
 * @file
 * Contains Drupal\yamlform\Form\YamlFormResultsExportForm.
 */

namespace Drupal\yamlform\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\yamlform\Controller\YamlFormResultsExportController;
use Drupal\yamlform\Utility\YamlFormHelper;
use Drupal\yamlform\YamlFormInterface;

/**
 * Form for YAML form results export form.
 */
class YamlFormResultsExportForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'yamlform_results_export';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, YamlFormInterface $yamlform = NULL) {
    $default_options = YamlFormResultsExportController::$defaultExportOptions;

    $form['format'] = [
      '#type' => 'details',
      '#title' => $this->t('Format options'),
      '#open' => TRUE,
    ];
    $form['format']['delimiter'] = [
      '#type' => 'select',
      '#title' => $this->t('Delimiter text format'),
      '#description' => $this->t('This is the delimiter used in the CSV/TSV file when downloading YAML form results. Using tabs in the export is the most reliable method for preserving non-latin characters. You may want to change this to another character depending on the program with which you anticipate importing results.'),
      '#options' => [
        ','  => $this->t('Comma (,)'),
        '\t' => $this->t('Tab (\t)'),
        ';'  => $this->t('Semicolon (;)'),
        ':'  => $this->t('Colon (:)'),
        '|'  => $this->t('Pipe (|)'),
        '.'  => $this->t('Period (.)'),
        ' '  => $this->t('Space ( )'),
      ],
      '#default_value' => $default_options['delimiter'],
    ];

    $form['format']['header_keys'] = [
      '#type' => 'radios',
      '#title' => $this->t('Column header format'),
      '#description' => $this->t('Choose whether to show the label or field key in each column header.'),
      '#options' => [
        'label' => $this->t('Input titles (label)'),
        'key' => $this->t('Input names (key)'),
      ],
      '#default_value' => $default_options['header_keys'],
    ];

    // Select options.
    $form['options'] = [
      '#type' => 'details',
      '#title' => $this->t('Select menu, radio buttons, and checkboxes options'),
      '#open' => TRUE,
    ];
    $form['options']['options_format'] = [
      '#type' => 'radios',
      '#title' => $this->t('Options format'),
      '#options' => [
        'compact' => $this->t('Compact; with the option values delimited by commas in one column.') . '<div class="description">' . $this->t('Compact options are more suitable for importing data into other systems.') . '</div>',
        'separate' => $this->t('Separate; with each possible option value in its own column.') . '<div class="description">' . $this->t('Separate options are more suitable for building reports, graphs, and statistics in a spreadsheet application.') . '</div>',
      ],
      '#default_value' => $default_options['options_format'],
    ];
    $form['options']['options_item_format'] = [
      '#type' => 'radios',
      '#title' => $this->t('Item format'),
      '#options' => [
        'label' => $this->t('Option labels, the human-readable value (label)'),
        'key' => $this->t('Option values, the raw value stored in the database (key)'),
      ],
      '#default_value' => $default_options['options_item_format'],
    ];

    // Entity reference options.
    $form['entity_reference'] = [
      '#type' => 'details',
      '#title' => $this->t('Entity reference options'),
      '#open' => TRUE,
    ];
    $form['entity_reference']['entity_reference_format'] = [
      '#type' => 'radios',
      '#title' => $this->t('Entity format'),
      '#options' => [
        'link' => $this->t('Entity link; with entity id, title and url in their own column.') . '<div class="description">' . $this->t("Entity links are suitable as long as there are not too many submissions (ie 1000's) pointing to just a few unique entities (ie 100's).") . '</div>',
        'id' => $this->t('Entity id; just the entity id column') . '<div class="description">' . $this->t('Entity links are suitable as long as there is mechanism for the referenced entity to be looked up external (ie REST API).') . '</div>',
      ],
      '#default_value' => $default_options['entity_reference_format'],
    ];

    // Inputs.
    $form['inputs'] = [
      '#type' => 'details',
      '#title' => t('Column options'),
    ];
    $form['inputs']['excluded_columns'] = [
      '#type' => 'yamlform_excluded_columns',
      '#description' => $this->t('The selected columns will be included in the export.'),
      '#yamlform' => $yamlform,
      '#default_value' => YamlFormResultsExportController::$defaultExportOptions['excluded_columns'],
    ];

    // Submission options.
    $form['submission'] = [
      '#type' => 'details',
      '#title' => $this->t('Submission options'),
      '#open' => TRUE,
    ];
    $form['submission']['limit'] = [
      '#type' => 'select',
      '#title' => $this->t('Number of submissions to download'),
      '#options' => [
        10 => 10,
        25 => 25,
        50 => 50,
        100 => 100,
        250 => 250,
        500 => 500,
        1000 => 1000,
        0 => $this->t('All'),
      ],
      '#default_value' => $default_options['limit'],
    ];

    // Download options.
    $form['download'] = [
      '#type' => 'details',
      '#title' => $this->t('Download options'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    $form['download']['download'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Download CSV'),
      '#default_value' => 1,
      '#description' => $this->t('If checked the CSV file will be automatically download to your local machine. If unchecked CSV file will be displayed as plain text within your browser.'),
    ];
    $form['download']['range_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Limit download to'),
      '#options' => [
        'all' => t('All'),
        // TODO: (OPTIONAL) Add support for 'new' which requires tracking last export.
        // 'new' => t('New'),
        'latest' => t('Latest'),
        'sid' => t('Submission ID'),
        'date' => t('Date'),
      ],
      '#default_value' => $default_options['range_type'],
    ];
    $form['download']['latest'] = [
      '#type' => 'container',
      '#attributes' => ['class' => 'container-inline'],
      '#states' => [
        'visible' => [
          ':input[name="download[range_type]"]' => ['value' => 'latest'],
        ],
      ],
      'range_latest' => [
        '#type' => 'number',
        '#title' => $this->t('Number of submissions'),
      ],
    ];
    $ranges = [
      'sid' => ['#type' => 'number'],
      'date' => ['#type' => 'date'],
    ];
    foreach ($ranges as $key => $range_element) {
      $form['download'][$key] = [
        '#type' => 'container',
        '#attributes' => ['class' => 'container-inline'],
        '#states' => [
          'visible' => [
            ':input[name="download[range_type]"]' => ['value' => $key],
          ],
        ],
      ];
      $form['download'][$key]['range_start'] = $range_element + [
        '#title' => $this->t('From'),
      ];
      $form['download'][$key]['range_end'] = $range_element + [
        '#title' => $this->t('To'),
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Download'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $values = YamlFormHelper::cleanupFormStateValues($values, ['submit']);

    // Exclude columns.
    $values['excluded_columns'] = implode(',', $values['excluded_columns']);

    // Download/Range type.
    if ($values['download']['range_type']) {
      $range_type = $values['download']['range_type'];
      $values['range_type'] = $range_type;
      if (isset($values['download'][$range_type])) {
        $values += $values['download'][$range_type];
      }
    }
    $values['download'] = $values['download']['download'];

    $route_parameters = ['yamlform' => $this->getRouteMatch()->getRawParameter('yamlform')];
    $route_options = ['query' => $values];
    $form_state->setRedirect('entity.yamlform.results_export', $route_parameters, $route_options);
  }

}
