<?php

/**
 * @file
 * Contains Drupal\yamlform\Form\YamlFormSubmissionsPurgeForm.
 */

namespace Drupal\yamlform\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\yamlform\YamlFormInterface;
use Drupal\yamlform\YamlFormSubmissionStorage;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for YAML form submission purge form.
 */
class YamlFormSubmissionsPurgeForm extends ConfirmFormBase {

  /**
   * The YAML form submission storage.
   *
   * @var \Drupal\yamlform\YamlFormSubmissionStorage
   */
  protected $submissionStorage;

  /**
   * Constructs a new YamlFormSubmissionPurgeForm object.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionStorage $yamlform_submission_storage
   *   The YAML form submission storage.
   */
  public function __construct(YamlFormSubmissionStorage $yamlform_submission_storage) {
    $this->submissionStorage = $yamlform_submission_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.manager')->getStorage('yamlform_submission')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'yamlform_results_clear';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete all submissions?');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Purge');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.yamlform_submission.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, YamlFormInterface $yamlform = NULL) {
    $form = parent::buildForm($form, $form_state);

    $submission_total = \Drupal::entityQuery('yamlform_submission')
      ->count()
      ->execute();

    $form_total = \Drupal::entityQuery('yamlform')
      ->count()
      ->execute();

    $t_args = [
      '@submission_total' => $submission_total,
      '@submissions' => $this->formatPlural($submission_total, $this->t('submission'), $this->t('submissions')),
      '@form_total' => $form_total,
      '@forms' => $this->formatPlural($form_total, $this->t('form'), $this->t('forms')),
    ];

    $form['confirm'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Are you sure you want to delete @submission_total @submissions in @form_total @forms?', $t_args),
      '#required' => TRUE,
      '#weight' => -10,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->submissionStorage->deleteAll();

    drupal_set_message($this->t('All form submissions cleared.'));
    $form_state->setRedirect('entity.yamlform.collection');
    $this->getRequest()->query->remove('destination');
  }

  /**
   * Get confirm purge message.
   *
   * This is message is used by this form and the
   * `drush yamlform-submission-purge` command.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup|string
   *   The confirm message with the total number of submissions and forms.
   */
  public static function getConfirmMessage() {
    $submission_total = \Drupal::entityQuery('yamlform_submission')
      ->count()
      ->execute();

    $form_total = \Drupal::entityQuery('yamlform')
      ->count()
      ->execute();

    $t_args = [
      '@submission_total' => $submission_total,
      '@submissions' => \Drupal::translation()->formatPlural($submission_total, t('submission'), t('submissions')),
      '@form_total' => $form_total,
      '@forms' => \Drupal::translation()->formatPlural($form_total, t('form'), t('forms')),
    ];

    return t('Are you sure you want to delete @submission_total @submissions in @form_total @forms?', $t_args);
  }

}
