<?php

/**
 * @file
 * Contains \Drupal\yamlform\Plugin\YamlFormElement\ManagedFile.
 */

namespace Drupal\yamlform\Plugin\YamlFormElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\yamlform\YamlFormElementBase;
use Drupal\Component\Utility\Bytes;
use Drupal\yamlform\YamlFormSubmissionInterface;

/**
 * Provides a 'managed_file' element.
 *
 * @YamlFormElement(
 *   id = "managed_file",
 *   label = @Translation("ManagedFile"),
 *   description = @Translation("Handles settings default properties, normalization, and displaying of a managed file element.")
 * )
 */
class ManagedFile extends YamlFormElementBase {

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    if (empty($element['#upload_location'])) {
      $element['#upload_location'] = 'public://yamlform/' . $yamlform_submission->getYamlForm()->id() . '/_sid_';
    }

    // Set max file size.
    $max_filesize = \Drupal::config('yamlform.settings')->get('inputs.default_max_filesize') ?: file_upload_max_size();
    $max_filesize = Bytes::toInt($max_filesize);
    if (!empty($element['#max_filesize'])) {
      $max_filesize = min($max_filesize, Bytes::toInt($element['#max_filesize']) * 1024);
    }
    $element['#upload_validators']['file_validate_size'] = [$max_filesize];

    // Set valid file extensions.
    $file_extensions = \Drupal::config('yamlform.settings')->get('inputs.default_file_extensions');
    if (!empty($element['#file_extensions'])) {
      $file_extensions = $element['#file_extensions'];
    }
    $element['#upload_validators']['file_validate_extensions'] = [$file_extensions];

    // Validate uploaded file.
    $element['#element_validate'][] = [get_class($this), 'validate'];
  }

  /**
   * {@inheritdoc}
   */
  public function setDefaultValue(array &$element, $default_value) {
    if (!empty($element['#default_value']) && !is_array($element['#default_value'])) {
      $element['#default_value'] = [$element['#default_value']];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function formatHtml(array &$element, $value) {
    return ($file = File::load($value)) ? \Drupal::service('link_generator')->generate($file->getFilename(), Url::fromUri(file_create_url($file->getFileUri()))) : '';
  }

  /**
   * {@inheritdoc}
   */
  public function formatText(array &$element, $value) {
    return ($file = File::load($value)) ? file_create_url($file->getFileUri()) : '';
  }

  /**
   * {@inheritdoc}
   */
  public function save(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    // Get current value and original value for this element.
    $key = $element['#key'];

    $original_data = $yamlform_submission->getOriginalData();
    $data =  $yamlform_submission->getData();

    $value = isset($data[$key]) ? $data[$key] : NULL;
    $original_value = isset($original_data[$key]) ? $original_data[$key] : NULL;

    // Check the original submission and delete the old file upload.
    if ($original_value && $original_value != $value) {
      file_delete($original_value);
    }

    $file = File::load($value);
    if (!$file) {
      return;
    }

    $source_uri = $file->getFileUri();

    // Replace /_sid_/ token with the submission id.
    if (strpos($source_uri, '/_sid_/')) {
      $destination_uri = str_replace('/_sid_/', '/' . $yamlform_submission->id() . '/', $source_uri);
      $destination_directory = drupal_dirname($destination_uri);
      file_prepare_directory($destination_directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);
      $destination_uri = file_unmanaged_move($source_uri, $destination_uri);
      // Update the file's uri and save.
      $file->setFileUri($destination_uri);
      $file->save();
    }

    // Update file usage table.
    // Set file usage which will also make the file's status permanent.
    /** @var \Drupal\file\FileUsage\FileUsageInterface $file_usage */
    $file_usage = \Drupal::service('file.usage');
    $file_usage->delete($file, 'yamlform', 'yamlform_submission', $yamlform_submission->id(), 0);
    $file_usage->add($file, 'yamlform', 'yamlform_submission', $yamlform_submission->id());
  }

  /**
   * {@inheritdoc}
   */
  public function postDelete(array &$element, YamlFormSubmissionInterface $yamlform_submission) {
    $yamlform = $yamlform_submission->getYamlForm();

    $data = $yamlform_submission->getData();
    $key = $element['#key'];

    // Delete managed file record.
    if (!empty($data[$key])) {
      file_delete($data[$key]);
    }

    // Remove the empty directory.
    file_unmanaged_delete_recursive('public://yamlform//' . $yamlform->id() . '/' . $yamlform_submission->id());
  }

  /**
   * Form API callback. Consolidate the array of fids for this field into a single fids.
   */
  public static function validate(array &$element, FormStateInterface $form_state) {
    if (!empty($element['#files'])) {
      $file = reset($element['#files']);
      $value = (int) $file->id();
    }
    else {
      $value = NULL;
    }
    $form_state->setValueForElement($element, $value);
  }

}
