<?php

/**
 * @file
 * Contains \Drupal\yamlform\Plugin\YamlFormHandler\EmailYamlFormHandler.
 */

namespace Drupal\yamlform\Plugin\YamlFormHandler;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Utility\Token;
use Drupal\yamlform\YamlFormHandlerBase;
use Drupal\yamlform\YamlFormSubmissionInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Emails a YAML form submission.
 *
 * @YamlFormHandler(
 *   id = "email",
 *   label = @Translation("Email"),
 *   description = @Translation("Sends a YAML form submission via an email."),
 *   cardinality = \Drupal\yamlform\YamlFormHandlerInterface::CARDINALITY_UNLIMITED,
 * )
 */
class EmailYamlFormHandler extends YamlFormHandlerBase {

  /**
   * A mail manager for sending email.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The token handler.
   *
   * @var \Drupal\Core\Utility\Token $token
   */
  protected $token;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerInterface $logger, MailManagerInterface $mail_manager, ConfigFactoryInterface $config_factory, Token $token) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger);
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')->get('yamlform'),
      $container->get('plugin.manager.mail'),
      $container->get('config.factory'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    return [
      '#settings' => $this->getEmailConfiguration(),
    ] + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'to_mail' => 'default',
      'from_mail' => 'default',
      'from_name' => 'default',
      'subject' => 'default',
      'body' => 'default',
      'excluded_inputs' => [],
      'debug' => FALSE,
    ];
  }

  /**
   * Get mail configuration values
   *
   * @return array
   *   An associative array containing email configuration values.
   */
  protected function getEmailConfiguration() {
    $configuration = $this->getConfiguration();
    $settings = $this->getConfigurationSettings();
    $email = [];
    foreach ($configuration['settings'] as $key => $value) {
      if ($value === 'default') {
        $email[$key] = $settings[$key]['default'];
      }
      else {
        $email[$key] = $value;
      }
    }
    return $email;
  }

  /**
   * Get configuration settings including type, label, and default value.
   *
   * @return array
   *   An associative array keyed by configuration name containing each
   *   configuration setting's type, label, and default value.
   */
  protected function getConfigurationSettings() {
    return [
      'to_mail' => [
        'type' => 'email',
        'label' => $this->t('Email to address'),
        'default' => \Drupal::config('system.site')->get('mail'),
      ],
      'from_mail' => [
        'type' => 'email',
        'label' => $this->t('Email from address'),
        'default' => \Drupal::config('system.site')->get('mail'),
      ],
      'from_name' => [
        'type' => 'textfield',
        'label' => $this->t('Email from name'),
        'default' => \Drupal::config('system.site')->get('name'),
      ],
      'subject' => [
        'type' => 'textfield',
        'label' => $this->t('Email subject'),
        'default' => $this->t('Form submission from: @form', ['@form' => $this->yamlform->label()]),
      ],
      'body' => [
        'type' => 'textarea',
        'label' => $this->t('Email body'),
        'default' => '{Default template}',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getConfigurationSettings();

    $form['to'] = [
      '#type' => 'details',
      '#title' => $this->t('Send to'),
      '#open' => TRUE,
    ];
    $form['from'] = [
      '#type' => 'details',
      '#title' => $this->t('Send from'),
      '#open' => TRUE,
    ];
    $form['message'] = [
      '#type' => 'details',
      '#title' => $this->t('Message'),
      '#open' => TRUE,
    ];

    $mail_input_options = [];
    $text_input_options = [];
    $inputs = $this->yamlform->getFlattenedInputs();
    foreach ($inputs as $key => $input) {
      $title = (isset($input['#title'])) ? new FormattableMarkup('@title (@key)', ['@title' => $input['#title'], '@key' => $key]) : $key;
      $token = "[yamlform-submission:values:$key]";
      if (in_array($input['#type'], ['email', 'hidden', 'value', 'selected', 'radios'])) {
        $mail_input_options[$token] = $title;
      }
      $text_input_options[$token] = $title;
    }

    foreach ($settings as $config_name => $config_settings) {
      $type = $config_settings['type'];
      $label = $config_settings['label'];
      $group = preg_match('/^(to|from)/', $config_name, $match) ? $match[0] : 'message';

      $inputs_optgroup = (string) $this->t('Inputs');

      // Set options
      $options = [
        'default' => $this->t('Default: @default', ['@default' => $config_settings['default']]),
        'custom' => $this->t('Custom...'),
      ];
      if ($type == 'email') {
        if ($mail_input_options) {
          $options[$inputs_optgroup] = $mail_input_options;
        }
        $custom_label = $this->t('Enter email address...');
      }
      else {
        if ($text_input_options) {
          $options[$inputs_optgroup] = $text_input_options;
        }
        $custom_label = $this->t('Enter text...');
      }

      $value = $this->configuration[$config_name];
      if (in_array($value, ['default', 'custom']) || isset($options[$inputs_optgroup][$value])) {
        $custom_value = '';
      }
      else {
        $custom_value = $value;
        $value = 'custom';
      }

      $form[$group][$config_name] = [
        '#type' => 'select',
        '#title' => $label,
        '#options' => $options,
        '#required' => TRUE,
        '#default_value' => $value,
      ];
      $form[$group][$config_name . '_custom'] = [
        '#type' => $type,
        '#title' => $this->t('@label custom', ['@label' => $label]),
        '#title_display' => 'hidden',
        '#default_value' => $custom_value,
        '#attributes' => ['placeholder' => $custom_label],
        '#states' => [
          'visible' => [
            ':input[name="settings[' . $group . '][' . $config_name . ']"]' => ['value' => 'custom'],
          ],
          'required' => [
            ':input[name="settings[' . $group . '][' . $config_name . ']"]' => ['value' => 'custom'],
          ],
        ],
      ];
    }

    // Inputs.
    $form['inputs'] = [
      '#type' => 'details',
      '#title' => $this->t('Included email values'),
      '#open' => $this->configuration['excluded_inputs'] ? TRUE : FALSE,
    ];
    $form['inputs']['excluded_inputs'] = [
      '#type' => 'yamlform_excluded_inputs',
      '#description' => $this->t('The selected inputs will be included in the [yamlform-submission:values] token. Individual values may still be printed if explicitly specified as a [yamlform-submission:values:?] in the email body template.'),
      '#yamlform' => $this->yamlform,
      '#default_value' => $this->configuration['excluded_inputs'],
    ];

    // Token.
    $form['message']['token_tree'] = [
      '#theme' => 'token_tree',
      '#token_types' => [
        'yamlform',
        'yamlform-submission',
      ],
      '#dialog' => TRUE,
    ];

    // DEBUG
    $form['debug'] = [
      '#type' => 'details',
      '#title' => $this->t('Debugging'),
      '#open' => $this->configuration['debug'] ? TRUE : FALSE,
    ];
    $form['debug']['debug'] = [
      '#type' => 'checkbox',
      '#title' => 'Enable debugging',
      '#description' => $this->t('If checked sent emails will be displayed onscreen to all users.'),
      '#default_value' => $this->configuration['debug'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    // Get email settings.
    $values = $form_state->getValue('to') + $form_state->getValue('from') + $form_state->getValue('message');
    foreach (['to_mail', 'from_mail', 'from_name', 'subject', 'body'] as $key) {
      $value = $values[$key];
      if ($value == 'custom') {
        $value = $values[$key . '_custom'];
      }
      $this->configuration[$key] = $value;
    }

    // Get other settings.
    $values = $form_state->getValues();
    $this->configuration['excluded_inputs'] = $values['inputs']['excluded_inputs'];
    $this->configuration['debug'] = $values['debug']['debug'];
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(YamlFormSubmissionInterface $yamlform_submission, $update = TRUE) {
    if (!$update) {
      $email = $this->getEmail($yamlform_submission);
      $this->mail($email);
    }
  }

  /**
   * Get a fully populated email for a YAML form submission.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   An array containing populated email parameters
   *   (subject, to_mail, from_mail, from_name, & body)
   */
  public function getEmail(YamlFormSubmissionInterface $yamlform_submission) {
    $token_data = [
      'yamlform' => $yamlform_submission->getYamlForm(),
      'yamlform-submission' => $yamlform_submission,
      'yamlform-submission-options' => [
        'excluded_inputs' => $this->configuration['excluded_inputs'],
      ],
    ];

    $email = $this->configuration;
    unset($email['excluded_inputs']);

    // Replace 'default' values and [tokens].
    $yamlform_settings = $this->configFactory->get('yamlform.settings');
    $site_settings = $this->configFactory->get('system.site');
    $default_values = [
      'subject' => $yamlform_settings->get('mail.default_subject') ?: 'Form submission from: [yamlform-submission:source-entity]',
      'to_mail' => $yamlform_settings->get('mail.default_to_mail') ?: $site_settings->get('mail') ?: ini_get('sendmail_from'),
      'from_mail' => $yamlform_settings->get('mail.default_from_mail') ?: $site_settings->get('mail') ?: ini_get('sendmail_from'),
      'from_name' => $yamlform_settings->get('mail.default_from_name') ?: $site_settings->get('name'),
      'body' => $yamlform_settings->get('mail.default_body') ?: '[yamlform-submission:values]',
    ];
    foreach ($default_values as $default_key => $default_value) {
      if (empty($email[$default_key]) || $email[$default_key] == 'default') {
        $email[$default_key] = $default_value;
      }
      $email[$default_key] = $this->token->replace($email[$default_key], $token_data);
    }

    // TODO: (API) Call email alter hook.

    // DEBUG:
    // dsm($email);

    return $email;
  }

  /**
   * Sends and logs a YAML form submission email.
   *
   * @param array $params
   *   An array of email parameter, including subject, to_mail, from_mail,
   *   from_name, and body.
   */
  public function mail(array $params) {
    // Send mail.
    $to = $params['to_mail'];
    $from = $params['from_mail'] . ' <' . $params['from_name'] . '>';
    $current_langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $this->mailManager->mail('yamlform', 'email.' . $this->getHandlerId(), $to, $current_langcode, $params, $from);

    // Log message.
    $variables = [
      '@from_name' => $params['from_name'],
      '@from_mail' => $params['from_mail'],
      '@to_mail' => $params['to_mail'],
      '@subject' => $params['subject'],
    ];
    \Drupal::logger('yamlform.email')->notice('@subject sent to @to_mail from @from_name [@from_mail].', $variables);

    // Debug by displaying send email onscreen.
    if ($this->configuration['debug']) {
      $t_args = [
        '%from_name' => $params['from_name'],
        '%from_mail' => $params['from_mail'],
        '%to_mail' => $params['to_mail'],
        '%subject' => $params['subject'],
      ];
      $build = [
        'message' => [
          '#markup' => $this->t('%subject sent to %to_mail from %from_name [%from_mail].', $t_args),
          '#prefix' => '<b>',
          '#suffix' => '</b>',
        ],
        'body' => [
          '#markup' => $params['body'],
          '#prefix' => '<pre>',
          '#suffix' => '</pre>',
        ],
      ];
      drupal_set_message(drupal_render($build), 'warning');
    }
  }

}
