<?php

/**
 * @file
 * Definition of Drupal\yamlform\test\YamlSubmissionFormTest.
 */

namespace Drupal\yamlform\Tests;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Session\AccountInterface;
use Drupal\yamlform\Entity\YamlForm;
use Drupal\yamlform\YamlFormSubmissionForm;

/**
 * Tests for YAML form submission form and inputs.
 *
 * @group YamlForm
 */
class YamlFormSubmissionFormTest extends YamlFormTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['system', 'block', 'filter', 'node', 'user', 'yamlform', 'yamlform_test'];

  /**
   * Tests prepare inputs.
   */
  public function testPrepareInputs() {
    // Check invalid inputs.
    $this->drupalGet('yamlform/test_inputs_invalid');
    $this->assertRaw('Unable to display this form. Please contact the site administrator.');

    // Check ignored properties.
    $yamlform_confirmation_inline = YamlForm::load('test_inputs_ignored_properties');
    $form = $yamlform_confirmation_inline->getSubmissionForm();
    foreach (YamlFormSubmissionForm::$ignoredProperties as $ignored_property) {
      $this->assert(!isset($form['inputs']['test'][$ignored_property]), new FormattableMarkup('@property ignored.', ['@property' => $ignored_property]));
    }

    // Check input with #private property hidden for normal user.
    $this->drupalLogin($this->normalUser);
    $this->drupalGet('yamlform/test_inputs_private');
    $this->assertNoFieldByName('private', '');

    // Check input with #private property visible for admin user.
    $this->drupalLogin($this->adminFormUser);
    $this->drupalGet('yamlform/test_inputs_private');
    $this->assertFieldByName('private', '');

    // TODO: (TESTING) Check element types.

    /* Test data elements */

    $yamlform_inputs_dates = YamlForm::load('test_inputs_dates');

    // Check 'text_format' values.
    $this->drupalGet('yamlform/test_inputs_dates');
    $this->assertFieldByName('date', '1969-08-18');
    $this->assertFieldByName('datetime[date]', '1969-08-18');
    $this->assertFieldByName('datetime[time]', '16:00:00');
    $this->assertFieldByName('datelist_date[month]', '8');

    // Check 'datelist' and 'datetime' #default_value.
    $form = $yamlform_inputs_dates->getSubmissionForm();
    $this->assert(is_string($form['inputs']['date']['#default_value']), 'date #default_value is a string.');
    $this->assert($form['inputs']['datetime']['#default_value'] instanceof DrupalDateTime, 'datelist_date #default_value instance of \Drupal\Core\Datetime\DrupalDateTime.');
    $this->assert($form['inputs']['datelist_date']['#default_value'] instanceof DrupalDateTime, 'datelist_date #default_value instance of \Drupal\Core\Datetime\DrupalDateTime.');

    // Check 'entity_autocomplete' #default_value.
    $yamlform_inputs_entity_autocomplete = YamlForm::load('test_inputs_entity_autocomplete');

    /* Test entity_autocomplete element */

    $this->drupalGet('yamlform/test_inputs_entity_autocomplete');
    $this->assertFieldByName('user', 'admin (1)');
    $this->assertFieldByName('users', '- Restricted access - (0), admin (1)');

    $form = $yamlform_inputs_entity_autocomplete->getSubmissionForm();

    // Single entity (w/o #tags).
    // TODO: (TESTING) Figure out why the below #default_value is an array when it should be the entity.
    // See: \Drupal\yamlform\YamlFormSubmissionForm::prepareInputs.
    $this->assert($form['inputs']['user']['#default_value'][0] instanceof AccountInterface, 'user #default_value instance of \Drupal\Core\Session\AccountInterface.');

    // Multiple entities (w #tags).
    $this->assert($form['inputs']['users']['#default_value'][0] instanceof AccountInterface, 'users #default_value instance of \Drupal\Core\Session\AccountInterface.');
    $this->assert($form['inputs']['users']['#default_value'][1] instanceof AccountInterface, 'users #default_value instance of \Drupal\Core\Session\AccountInterface.');

    /* Test text format element */

    $yamlform_test_inputs_text_format = YamlForm::load('test_inputs_text_format');

    // Check 'text_format' values.
    $this->drupalGet('yamlform/test_inputs_text_format');
    $this->assertFieldByName('text_format[value]', 'The quick brown fox jumped over the lazy dog.');
    $this->assertRaw('No HTML tags allowed.');

    $text_format = [
      'value' => 'Custom value',
      'format' => 'custom_format',
    ];
    $form = $yamlform_test_inputs_text_format->getSubmissionForm(['data' => ['text_format' => $text_format]]);
    $this->assertEqual($form['inputs']['text_format']['#default_value'], $text_format['value']);
    $this->assertEqual($form['inputs']['text_format']['#format'], $text_format['format']);
  }

  /**
   * Tests YAML form setting including confirmation.
   */
  public function testSettings() {
    $this->drupalLogin($this->adminFormUser);

    /* Test confirmation message (confirmation_type=message) */

    // Check confirmation message.
    $this->drupalPostForm('yamlform/test_confirmation_message', [], t('Submit'));
    $this->assertRaw('This is a custom confirmation message.');
    $this->assertUrl('yamlform/test_confirmation_message');

    // Check confirmation page with custom query parameters.
    $this->drupalPostForm('yamlform/test_confirmation_message', [], t('Submit'), ['query' => ['custom' => 'param']]);
    $this->assertUrl('yamlform/test_confirmation_message', ['query' => ['custom' => 'param']]);

    /* Test confirmation inline (confirmation_type=inline) */

    $yamlform_confirmation_inline = YamlForm::load('test_confirmation_inline');

    // Check confirmation inline.
    $this->drupalPostForm('yamlform/test_confirmation_inline', [], t('Submit'));
    $this->assertRaw('This is a custom inline confirmation message.');
    $this->assertRaw('<a href="' . $yamlform_confirmation_inline->toUrl()->toString() . '" data-drupal-selector="edit-back-to" id="edit-back-to">Back to form</a>');
    $this->assertUrl('yamlform/test_confirmation_inline', ['query' => ['yamlform_id' => $yamlform_confirmation_inline->id()]]);

    // Check confirmation inline with custom query parameters.
    $this->drupalPostForm('yamlform/test_confirmation_inline', [], t('Submit'), ['query' => ['custom' => 'param']]);
    $this->assertRaw('<a href="' . $yamlform_confirmation_inline->toUrl()->toString() . '?custom=param" data-drupal-selector="edit-back-to" id="edit-back-to">Back to form</a>');
    $this->assertUrl('yamlform/test_confirmation_inline', ['query' => ['custom' => 'param', 'yamlform_id' => $yamlform_confirmation_inline->id()]]);

    /* Test confirmation page (confirmation_type=page) */

    $yamlform_confirmation_page = YamlForm::load('test_confirmation_page');

    // Check confirmation page.
    $this->drupalPostForm('yamlform/test_confirmation_page', [], t('Submit'));
    $this->assertRaw('This is a custom confirmation page.');
    $this->assertRaw('<a href="' . $yamlform_confirmation_page->toUrl()->toString() . '">Back to form</a>');
    $this->assertUrl('yamlform/test_confirmation_page/confirmation');

    // Check that the confirmation page's 'Back to form 'link includes custom
    // query parameters.
    $this->drupalGet('yamlform/test_confirmation_page/confirmation', ['query' => ['custom' => 'param']]);

    // Check confirmation page with custom query parameters.
    $this->drupalPostForm('yamlform/test_confirmation_page', [], t('Submit'), ['query' => ['custom' => 'param']]);
    $this->assertUrl('yamlform/test_confirmation_page/confirmation', ['query' => ['custom' => 'param']]);
    // TODO: (TESTING)  Figure out why the inline confirmation link is not including the query string parameters.
    // $this->assertRaw('<a href="' . $yamlform_confirmation_page->toUrl()->toString() . '?custom=param">Back to form</a>');

    /* Test confirmation URL (confirmation_type=url) */

    // Check confirmation URL.
    $this->drupalPostForm('yamlform/test_confirmation_url', [], t('Submit'));
    $this->assertRaw('This is a custom confirmation message.');
    $this->assertUrl('<front>');

    /* Test form closed (status=false)*/

    $yamlform_form_closed = YamlForm::load('test_form_closed');

    // Check form closed message is displayed.
    $this->assertTrue($yamlform_form_closed->isClosed());
    $this->assertFalse($yamlform_form_closed->isOpen());
    $this->drupalGet('yamlform/test_form_closed');
    $this->assertNoRaw('This message should not be displayed)');
    $this->assertRaw('This form is closed.');

    // Check form closed message is displayed.
    $yamlform_form_closed->setSettings(['form_closed_message' => '']);
    $yamlform_form_closed->save();
    $this->drupalGet('yamlform/test_form_closed');
    $this->assertNoRaw('This form is closed.');
    $this->assertRaw('Sorry...This form is closed to new submissions.');

    // Check form closed message is not displayed.
    $yamlform_form_closed->set('status', 1);
    $yamlform_form_closed->save();
    $this->assertFalse($yamlform_form_closed->isClosed());
    $this->assertTrue($yamlform_form_closed->isOpen());
    $this->drupalGet('yamlform/test_form_closed');
    $this->assertRaw('This message should not be displayed');
    $this->assertNoRaw('This form is closed.');

    /* Test form prepopulate (form_prepopulate) */

    $yamlform_form_prepopulate = YamlForm::load('test_form_prepopulate');

    // Check prepopulation of an input.
    $this->drupalGet('yamlform/test_form_prepopulate', ['query' => ['name' => 'John']]);
    $this->assertFieldByName('name', 'John');

    // Check disabling prepopulation of an input.
    $yamlform_form_prepopulate->setSettings(['form_prepopulate' => FALSE]);
    $yamlform_form_prepopulate->save();
    $this->drupalGet('yamlform/test_form_prepopulate', ['query' => ['name' => 'John']]);
    $this->assertFieldByName('name', '');

    /* Test form preview (form_preview) */

    $yamlform_preview = YamlForm::load('test_preview');

    // Check form with optional preview.
    $this->drupalGet('yamlform/test_preview');
    $this->assertFieldByName('op', 'Submit');
    $this->assertFieldByName('op', 'Preview');

    // Check default preview.
    $this->drupalPostForm('yamlform/test_preview', ['name' => 'test'], t('Preview'));

    $this->assertRaw('Please review your submission. Your submission is not complete until you press the &quot;Submit&quot; button!');
    $this->assertFieldByName('op', 'Submit');
    $this->assertFieldByName('op', '< Previous');
    $this->assertPattern('#<label for="edit-preview-name">Name</label>\s+test#s');

    // Check required preview with custom settings.
    $yamlform_preview->setSettings([
      'preview' => DRUPAL_REQUIRED,
      'preview_next_button_label' => '{Preview}',
      'preview_prev_button_label' => '{Back}',
      'preview_message' => '{Message}',
    ]);
    $yamlform_preview->save();

    // Check custom preview.
    $this->drupalPostForm('yamlform/test_preview', ['name' => 'test'], t('{Preview}'));
    $this->assertRaw('{Message}');
    $this->assertFieldByName('op', 'Submit');
    $this->assertFieldByName('op', '{Back}');
    $this->assertPattern('#<label for="edit-preview-name">Name</label>\s+test#s');

    $this->drupalGet('yamlform/test_preview');
    $this->assertNoFieldByName('op', 'Submit');
    $this->assertFieldByName('op', '{Preview}');

    /* Test results disabled (results_disabled=true) */

    // Check results disabled.
    $yamlform_results_disabled = YamlForm::load('test_results_disabled');
    $submission = $this->postSubmission($yamlform_results_disabled);
    $this->assertFalse($submission, 'Submission not saved to the database.');

    // Check error message form admins.
    $this->drupalGet('yamlform/test_results_disabled');
    $this->assertRaw(t('This form is currently not saving any submitted data.'));
    $this->assertFieldByName('op', t('Submit'));

    // Check form disable for everyone else.
    $this->drupalLogout();
    $this->drupalGet('yamlform/test_results_disabled');
    $this->assertNoRaw(t('This form is currently not saving any submitted data.'));
    $this->assertNoFieldByName('op', t('Submit'));
    $this->assertRaw(t('This form is unable to process submissions. Please contact the site administrator.'));

  }

}
