<?php

/**
 * @file
 * Definition of Drupal\yamlform\Tests\YamlFormTestBase.
 */

namespace Drupal\yamlform\Tests;

use Drupal\Component\Serialization\Yaml;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\Unicode;
use Drupal\Component\Utility\UrlHelper;
use Drupal\simpletest\WebTestBase;
use Drupal\yamlform\Entity\YamlForm;
use Drupal\yamlform\Entity\YamlFormSubmission;
use Drupal\yamlform\YamlFormInterface;

/**
 * Defines an abstract test base for YAML form tests.
 */
abstract class YamlFormTestBase extends WebTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['system', 'block', 'node', 'user', 'yamlform', 'yamlform_test'];

  /**
   * A normal user to submit forms.
   */
  protected $normalUser;

  /**
   * An YAML form administrator.
   */
  protected $adminFormUser;

  /**
   * An YAML form submission administrator.
   */
  protected $adminSubmissionUser;

  /**
   * Associative array of nodes keyed by nid.
   *
   * @var array
   */
  protected $nodes = [];

  /**
   * YAML form submission storage.
   *
   * @var \Drupal\yamlform\YamlFormSubmissionStorage
   */
  protected $submissionStorage;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    // Place breadcrumb page, tasks, and actions.
    $this->drupalPlaceBlock('system_breadcrumb_block');
    $this->drupalPlaceBlock('page_title_block');
    $this->drupalPlaceBlock('local_tasks_block');
    $this->drupalPlaceBlock('local_actions_block');

    // Create users.
    $this->normalUser = $this->drupalCreateUser(['access user profiles', 'access content', 'access yamlform']);;
    $this->adminFormUser = $this->drupalCreateUser(['access user profiles', 'access content', 'administer yamlform', 'administer blocks', 'administer nodes', 'administer users']);
    $this->adminSubmissionUser = $this->drupalCreateUser(['access user profiles', 'access content', 'administer yamlform submission']);

    // Storage.
    $this->submissionStorage = \Drupal::entityManager()->getStorage('yamlform_submission');

    // Set page.front (aka <front>) to /node instead of /user/login.
    \Drupal::configFactory()->getEditable('system.site')->set('page.front', '/node')->save();

    // Login the normal user.
    $this->drupalLogin($this->normalUser);
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown() {
    // Must purge all submission before the yamlform.module is uninstalled.
    db_query('DELETE FROM {yamlform_submission}');
    parent::tearDown();
  }

  /**
   * Post a new submission to a node.
   *
   * @param \Drupal\yamlform\YamlFormInterface $yamlform
   *   A YAML form.
   * @param array $edit
   *   Submission values.
   * @param string $submit
   *   Value of the submit button whose click is to be emulated.
   *
   * @return int $sid
   *   The created submission's sid.
   */
  protected function postSubmission(YamlFormInterface $yamlform, array $edit = [], $submit = NULL) {
    $submit = $submit ?: t('Submit');

    // Post a new submission.
    $this->drupalPostForm('yamlform/' . $yamlform->id(), $edit, $submit);

    // Get submission sid.
    $url = UrlHelper::parse($this->getUrl());
    if (isset($url['query']['sid'])) {
      return $url['query']['sid'];
    }
    else {
      $entity_ids = \Drupal::entityQuery('yamlform_submission')
        ->sort('sid', 'DESC')
        ->condition('yamlform_id', $yamlform->id())
        ->execute();
      return reset($entity_ids);
    }
  }

  /**
   * Get nodes keyed by nid.
   *
   * @return \Drupal\node\NodeInterface[]
   *   Associative array of nodes keyed by nid.
   */
  protected function getNodes() {
    if (empty($this->nodes)) {
      for ($i = 0; $i < 3; $i++) {
        $this->nodes[$i] = $this->drupalCreateNode(['title' => 'Node ' . $i, 'status' => NODE_PUBLISHED]);
      }
    }
    return $this->nodes;
  }

  /**
   * Create a YAML form with submissions.
   *
   * @return \Drupal\yamlform\Entity\YamlForm
   *   A YAML form.
   */
  protected function createYamlForm($inputs = NULL, array $settings = []) {
    if ($inputs === NULL) {
      $inputs = [
        'first_name' => [
          '#type' => 'textfield',
          '#title' => 'First name',
        ],
        'last_name' => [
          '#type' => 'textfield',
          '#title' => 'Last name',
        ],
        'sex' => [
          '#type' => 'select',
          '#title' => 'Sex',
          '#options' => 'gender',
        ],
        'dob' => [
          '#type' => 'date',
          '#title' => 'Date of birth',
          '#date_format' => 'l, F j, Y',
        ],
        'node' => [
          '#type' => 'entity_autocomplete',
          '#title' => 'Favorite node',
          '#target_type' => 'node',
        ],
        'colors' => [
          '#type' => 'checkboxes',
          '#title' => 'Flag colors',
          '#options' => [
            'red' => 'Red',
            'white' => 'White',
            'blue' => 'Blue',
          ],
        ],
      ];
    }

    // Create new form.
    $id = $this->randomMachineName(8);
    $yamlform = YamlForm::create([
      'langcode' => 'en',
      'status' => TRUE,
      'id' => $id,
      'title' => $id,
      'inputs' => Yaml::encode($inputs),
      'settings' => $settings + [
        'form_submit_label' => '',
        'form_closed_message' => '',
        'confirmation_type' => 'page',
        'confirmation_message' => '',
        'confirmation_url' => '',
        'results_disabled' => FALSE,
      ],
    ]);
    $yamlform->save();
    return $yamlform;
  }

  /**
   * Create a YAML form with submissions.
   *
   * @return array
   *   Array containing the YAML form and submissions.
   */
  protected function createYamlFormWithSubmissions() {
    $yamlform = $this->createYamlForm();

    $nodes = $this->getNodes();

    // Create some submissions.
    $names = [
      ['George', 'Washington', 'Male', '1732-02-22', $nodes[0], ['white']],
      ['Abraham', 'Lincoln', 'Male', '1809-02-12', $nodes[1], ['red', 'white', 'blue']],
      ['Hillary', 'Clinton', 'Female', '1947-10-26', $nodes[2], ['red']],
    ];
    $sids = [];
    foreach ($names as $key => $name) {
      $edit = [
        'first_name' => $name[0],
        'last_name' => $name[1],
        'sex' => $name[2],
        'dob' => $name[3],
        'node' => $name[4]->label() . '(' . $name[4]->id() . ')',
      ];
      foreach ($name[5] as $color) {
        $edit["colors[$color]"] = $color;
      }
      $sids[] = $this->postSubmission($yamlform, $edit);
    }

    // Change array keys to index instead of using entity ids.
    $submissions = array_values(YamlFormSubmission::loadMultiple($sids));

    $this->assert($yamlform instanceof YamlForm, 'YamlForm was created');
    $this->assertEqual(count($submissions), 3, 'YamlFormSubmissions were created.');

    return [$yamlform, $submissions];
  }

  /****************************************************************************/
  // Debug and custom assert methods
  /****************************************************************************/

  /**
   * Passes if the substring is contained within text, fails otherwise.
   */
  protected function assertContains($haystack, $needle, $message = '', $group = 'Other') {
    if (!$message) {
      $t_args = [
        '@haystack' => Unicode::truncate($haystack, 150, TRUE, TRUE),
        '@needle' => $needle,
      ];
      $message = new FormattableMarkup('"@needle" found', $t_args);
    }
    $result = (strpos($haystack, $needle) !== FALSE);
    if (!$result) {
      $this->verbose($haystack);
    }
    return $this->assert($result, $message, $group);
  }

  /**
   * Passes if the substring is not contained within text, fails otherwise.
   */
  protected function assertNotContains($haystack, $needle, $message = '', $group = 'Other') {
    if (!$message) {
      $t_args = [
        '@haystack' => Unicode::truncate($haystack, 150, TRUE, TRUE),
        '@needle' => $needle,
      ];

      $message = new FormattableMarkup('"@needle" not found', $t_args);
    }
    $result = (strpos($haystack, $needle) === FALSE);
    if (!$result) {
      $this->verbose($haystack);
    }
    return $this->assert($result, $message, $group);
  }

  /**
   * Logs verbose (debug) message in a text file.
   *
   * @param mixed $data
   *   Data to be output.
   */
  protected function debug($data) {
    $string = var_export($data, TRUE);
    $string = preg_replace('/=>\s*array\s*\(/', '=> array(', $string);
    $this->verbose('<pre>' . htmlentities($string) . '</pre>');
  }

}
