<?php
/**
 * @file
 * Contains Drupal\yamlform\YamlFormEntityListBuilder.
 */

namespace Drupal\yamlform;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;

/**
 * Defines a class to build a listing of YAML form entities.
 *
 * @see \Drupal\yamlform\Entity\YamlForm
 */
class YamlFormEntityListBuilder extends ConfigEntityListBuilder {

  /**
   * Search keys.
   *
   * @var string
   */
  protected $keys;

  /**
   * YAML form submission storage.
   *
   * @var \Drupal\yamlform\YamlFormSubmissionStorage
   */
  protected $submissionStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage) {
    parent::__construct($entity_type, $storage);
    $this->keys = \Drupal::request()->query->get('search');
    $this->submissionStorage = \Drupal::entityManager()->getStorage('yamlform_submission');
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    // Add the filter.
    $build['filter_form'] = \Drupal::formBuilder()->getForm('\Drupal\yamlform\Form\YamlFormFilterForm', $this->t('forms'), $this->keys);

    // Display info.
    if ($total = $this->getTotal()) {
      $t_args = [
        '@total' => $total,
        '@results' => $this->formatPlural($total, $this->t('form'), $this->t('forms')),
      ];
      $build['info'] = [
        '#markup' => $this->t('@total @results', $t_args),
        '#prefix' => '<div>',
        '#suffix' => '</div>',
      ];
    }

    $build += parent::render();
    return $build;
  }
  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['title'] = $this->t('Title');
    $header['description'] = $this->t('Description');
    $header['status'] = $this->t('Status');
    $header['results_status'] = $this->t('Results status');
    $header['results_total'] = $this->t('Results total');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /* @var $entity \Drupal\yamlform\YamlFormInterface */
    $settings = $entity->getSettings();

    $row['title'] = $entity->toLink();
    $row['description'] = $entity->get('description');
    $row['status'] = $entity->isOpen() ? $this->t('Open') : $this->t('Closed');
    $row['results_status'] = (!empty($settings['results_disabled'])) ? $this->t('Disabled') : $this->t('Enabled');
    $row['results_total'] = $this->submissionStorage->getTotal($entity);
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = parent::getDefaultOperations($entity);

    $route_parameters = ['yamlform' => $entity->id()];

    $operations['view'] = [
      'title' => $this->t('View'),
      'weight' => 20,
      'url' => Url::fromRoute('entity.yamlform.canonical', $route_parameters),
    ];

    $operations['test'] = [
      'title' => $this->t('Test'),
      'weight' => 21,
      'url' => Url::fromRoute('entity.yamlform.test', $route_parameters),
    ];

    $operations['results'] = [
      'title' => $this->t('Results'),
      'weight' => 22,
      'url' => Url::fromRoute('entity.yamlform.results_submissions', $route_parameters),
    ];

    $operations['export'] = [
      'title' => $this->t('Export'),
      'weight' => 23,
      'url' => Url::fromRoute('entity.yamlform.results_export', $route_parameters),
    ];

    $operations['clear'] = [
      'title' => $this->t('Clear'),
      'weight' => 24,
      'url' => Url::fromRoute('entity.yamlform.results_clear', $route_parameters),
    ];

    if ($entity->access('duplicate') && $entity->hasLinkTemplate('duplicate-form')) {
      $operations['duplicate'] = [
        'title' => $this->t('Duplicate'),
        'weight' => 25,
        'url' => Url::fromRoute('entity.yamlform.duplicate_form', $route_parameters),
      ];
    }
    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    return $this->getQuery()
      ->sort('title')
      ->pager($this->limit)
      ->execute();
  }

  /**
   * Get the total number of submissions.
   *
   * @return int
   *   The total number of submissions.
   */
  protected function getTotal() {
    return $this->getQuery()
      ->count()
      ->execute();
  }

  /**
   * Get the base entity query filtered by YAML form and search.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   An entity query.
   */
  protected function getQuery() {
    $query = $this->getStorage()->getQuery();

    // Filter forms
    if ($this->keys) {
      $or = $query->orConditionGroup()
        ->condition('title', $this->keys, 'CONTAINS')
        ->condition('inputs', $this->keys, 'CONTAINS');
      $query->condition($or);
    }

    return $query;
  }

}
