<?php

/**
 * @file
 * Contains Drupal\yamlform\YamlFormEntitySettingsForm.
 */

namespace Drupal\yamlform;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Base for controller for YAML form settings.
 */
class YamlFormEntitySettingsForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\yamlform\YamlFormInterface $yamlform */
    $yamlform = $this->entity;

    $settings = $yamlform->getSettings();
    $default_settings = $this->config('yamlform.settings')->get('settings');

    $form['form'] = [
      '#type' => 'details',
      '#title' => $this->t('Form settings'),
      '#open' => TRUE,
    ];
    $form['form']['status'] = [
      '#type' => 'radios',
      '#title' => $this->t('Form status'),
      '#default_value' => ($yamlform->get('status') == 1) ? 1 : 0,
      '#description' => $this->t('Closing a form prevents any further submissions by any users.'),
      '#options' => [
        1 => $this->t('Open'),
        0 => $this->t('Closed'),
      ],
      '#required' => TRUE,
      '#parents' => ['status'],
    ];
    $form['form']['form_closed_message']  = [
      '#type' => 'textarea',
      '#title' => $this->t('Closed message'),
      '#description' => $this->t('A message to be displayed notifying the user that the form is closed.') . '<br/>' .
        $this->t('Defaults to: %value', ['%value' => $default_settings['default_form_closed_message']]),
      "#rows" => 6,
      '#default_value' => $settings['form_closed_message'],
    ];
    $form['form']['form_submit_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Submit button label'),
      '#description' => $this->t('Defaults to: %value', ['%value' => $default_settings['default_form_submit_label']]),
      '#size' => 20,
      '#default_value' => $settings['form_submit_label'],
    ];
    $form['form']['form_prepopulate'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow inputs to be populated using query string parameters.'),
      '#default_value' => $settings['form_prepopulate'],
    ];

    $form['preview'] = [
      '#type' => 'details',
      '#title' => $this->t('Preview page'),
      '#open' => TRUE,
    ];
    $form['preview']['preview'] = [
      '#type' => 'radios',
      '#title' => $this->t('Enable preview page'),
      '#options' => [
        DRUPAL_DISABLED => $this->t('Disabled'),
        DRUPAL_OPTIONAL => $this->t('Optional'),
        DRUPAL_REQUIRED => $this->t('Required'),
      ],
      '#description' => $this->t('Add a page for previewing the form before submitting.'),
      '#default_value' => $settings['preview'],
    ];
    $form['preview']['settings'] = [
      '#type' => 'container',
      '#states' => [
        'invisible' => [
          ':input[name="preview"]' => ['value' => DRUPAL_DISABLED],
        ],
      ],
    ];
    $form['preview']['settings']['preview_next_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Preview button label'),
      '#description' => $this->t('The text for the button that will proceed to the preview page.') . '<br/>' .
        $this->t('Defaults to: %value', ['%value' => $default_settings['default_preview_next_button_label']]),
      '#size' => 20,
      '#default_value' => $settings['preview_next_button_label'],
    ];
    $form['preview']['settings']['preview_prev_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Previous page button label'),
      '#description' => $this->t('The text for the button to go backwards from the preview page.') . '<br/>' .
        $this->t('Defaults to: %value', ['%value' => $default_settings['default_preview_prev_button_label']]),
      '#size' => 20,
      '#default_value' => $settings['preview_prev_button_label'],
    ];
    $form['preview']['settings']['preview_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Preview message'),
      '#description' => $this->t('A message to be displayed on the preview page.') . '<br/>' .
        $this->t('Defaults to: %value', ['%value' => $default_settings['default_preview_message']]),
      '#default_value' => $settings['preview_message'],
    ];

    $form['confirmation'] = [
      '#type' => 'details',
      '#title' => $this->t('Confirmation settings'),
      '#open' => TRUE,
    ];
    $form['confirmation']['confirmation_type'] = [
      '#title' => $this->t('Confirmation type'),
      '#type' => 'radios',
      '#options' => [
        'page' => $this->t('Page (redirects to new page)'),
        'inline' => $this->t('Inline (reloads the current page and replaces the form with the confirmation message.)'),
        'message' => $this->t('Message (reloads the current page and displays the confirmation message as the top of the page.)'),
        'url' => $this->t('URL (redirects to a custom path or URL)'),
      ],
      '#default_value' => $settings['confirmation_type'],
    ];

    $form['confirmation']['confirmation_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Confirmation message'),
      '#description' => $this->t('Message to be shown upon successful submission. If the confirmation type is set to <em>Page</em> it will be shown on its own page, otherwise this displays as a message.'),
      '#default_value' => $settings['confirmation_message'],
    ];

    $form['confirmation']['confirmation_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Confirmation URL'),
      '#description' => $this->t('URL to redirect the user to upon successful submission.'),
      '#default_value' => $settings['confirmation_url'],
      '#states' => [
        'visible' => [
          ':input[name="settings[confirmation][confirmation_type]"]' => ['value' => 'url'],
        ],
      ],
    ];

    $form['results'] = [
      '#type' => 'details',
      '#title' => $this->t('Results settings'),
      '#open' => TRUE,
    ];
    $form['results']['results_disabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disable saving of submissions.'),
      '#description' => $this->t('If results are disabled, submissions must be sent via email.'),
      '#default_value' => $settings['results_disabled'],
    ];

    return parent::form($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function actionsElement(array $form, FormStateInterface $form_state) {
    $element = parent::actionsElement($form, $form_state);
    // Don't display delete button.
    unset($element['delete']);
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    /** @var \Drupal\yamlform\YamlFormInterface $yamlform */
    $yamlform = $this->getEntity();

    // Status is not part the YAML form settings, so it needs to be set and
    // removed from the values array.
    $yamlform->set('status', $values['status']);
    unset($values['status']);

    $yamlform->setSettings($values);

    $yamlform->save();

    $this->logger('yamlform')->notice('YAML form settings @label saved.', ['@label' => $yamlform->label()]);
    drupal_set_message($this->t('YAML form settings %label saved.', ['%value' => $yamlform->label()]));
  }

}
