<?php

/**
 * @file
 * Contains \Drupal\yamlform\YamlFormInterface.
 */

namespace Drupal\yamlform;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityWithPluginCollectionInterface;

/**
 * Provides an interface defining a YAML form entity.
 */
interface YamlFormInterface extends ConfigEntityInterface, EntityWithPluginCollectionInterface {

  /**
   * Determine if the form's inputs include a managed_file upload element.
   *
   * @return bool
   *   TRUE if the form's inputs include a managed_file upload element.
   */
  public function hasManagedFile();

  /**
   * Returns the YAML form opened status indicator.
   *
   * @return bool
   *   TRUE if the YAML form is open to new submissions.
   */
  public function isOpen();

  /**
   * Returns the YAML form closed status indicator.
   *
   * @return bool
   *   TRUE if the YAML form is closed to new submissions.
   */
  public function isClosed();

  /**
   * Returns the YAML form settings.
   *
   * @return array
   *   A structured array containing all the YAML form settings.
   */
  public function getSettings();

  /**
   * Sets the YAML form settings.
   *
   * @param array $settings
   *   The structured array containing all the YAML form settings settings.
   *
   * @return $this
   */
  public function setSettings(array $settings);

  /**
   * Returns the YAML form default settings.
   *
   * @return array
   *   A structured array containing all the YAML form default settings.
   */
  public function getDefaultSettings();

  /**
   * Get YAML form submission form.
   *
   * @param array $values
   *   (optional) An array of values to set, keyed by property name.
   *
   * @return array
   *   A render array representing a YAML form submission form.
   */
  public function getSubmissionForm(array $values = []);

  /**
   * Get inputs (YAML) as an associative array.
   *
   * @return array|bool
   *   Inputs as an associative array. Returns FALSE is inputs YAML is invalid.
   */
  public function getInputs();

  /**
   * Get YAML form inputs flattened into an associative array.
   *
   * @return array
   *   YAML form inputs flattened into an associative array keyed by input name.
   */
  public function getFlattenedInputs();

  /**
   * Get YAML form inputs flattened into an associative array of columns.
   *
   * @return array
   *   YAML form inputs flattened into an associative array of columns
   *   keyed by input name.
   */
  public function getInputColumns();

  /**
   * Get YAML form element options.
   *
   * @return array
   *   An associative array of options.
   */
  public function getElementOptions(array $element);

  /**
   * Returns a specific YAML form handler.
   *
   * @param string $effect
   *   The YAML form handler ID.
   *
   * @return \Drupal\yamlform\YamlFormHandlerInterface
   *   The YAML form handler object.
   */
  public function getHandler($effect);

  /**
   * Returns the YAML form handlers for this YAML form.
   *
   * @param string $plugin_id
   *   (optional) Plugin id used to return specific plugin instances (ie handlers).
   *
   * @return \Drupal\yamlform\YamlFormHandlerPluginCollection|\Drupal\yamlform\YamlFormHandlerInterface[]
   *   The YAML form handler plugin collection.
   */
  public function getHandlers($plugin_id = NULL);

  /**
   * Saves an YAML form handler for this YAML form.
   *
   * @param array $configuration
   *   An array of YAML form handler configuration.
   *
   * @return string
   *   The YAML form handler ID.
   */
  public function addYamlFormHandler(array $configuration);

  /**
   * Deletes an YAML form handler from this style.
   *
   * @param \Drupal\yamlform\YamlFormHandlerInterface $effect
   *   The YAML form handler object.
   *
   * @return $this
   */
  public function deleteYamlFormHandler(YamlFormHandlerInterface $effect);

  /**
   * Invoke a handlers method.
   *
   * @param string $method
   *   The handle method to be invoked.
   * @param mixed $data
   *   The argument to passed by reference to the handler method.
   */
  public function invokeHandlers($method, &$data, &$context1 = NULL, &$context2 = NULL);

  /**
   * Required to allow YAML form which are config entities to have an EntityViewBuilder.
   *
   * Prevents:
   *   Fatal error: Call to undefined method
   *   Drupal\yamlform\Entity\YamlForm::isDefaultRevision()
   *   in /private/var/www/sites/d8_dev/core/lib/Drupal/Core/Entity/EntityViewBuilder.php
   *   on line 169
   *
   * @see: \Drupal\Core\Entity\RevisionableInterface::isDefaultRevision
   *
   * @return bool
   *   Always return TRUE since config entities are not revisionable.
   */
  public function isDefaultRevision();

}
