<?php
/**
 * @file
 * Contains Drupal\yamlform\YamlFormSubmissionForm.
 */

namespace Drupal\yamlform;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\yamlform\Access\YamlFormAccess;
use Drupal\yamlform\Controller\YamlFormController;
use Drupal\yamlform\Utility\YamlFormHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base for controller for YAML form submission forms.
 */
class YamlFormSubmissionForm extends ContentEntityForm {

  /**
   * An associative array containing ignored input element properties.
   *
   * @var array
   */
  public static $ignoredProperties = [
    // Test value(s). See \Drupal\yamlform\Controller\YamlFormTestController.
    '#test' => '#test',
    '#tests' => '#tests',
    // Properties that will break YAML form data handling.
    '#tree' => '#tree',
    '#array_parents' => '#array_parents',
    // Properties that will cause unpredictable rendering.
    '#weight' => '#weight',
    // Callbacks are blocked to prevent unwanted code executions.
    '#after_build' => '#after_build',
    '#element_validate' => '#element_validate',
    '#post_render' => '#post_render',
    '#pre_render' => '#pre_render',
    '#process' => '#process',
    '#submit' => '#submit',
    '#validate' => '#validate',
    '#value_callback' => '#value_callback',
  ];

  /**
   * The YAML form element (plugin) manager.
   *
   * @var \Drupal\yamlform\YamlFormElementManager
   */
  protected $elementManager;
  /**
   * Constructs a ContentEntityForm object.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   */
  public function __construct(EntityManagerInterface $entity_manager, YamlFormElementManager $element_manager) {
    parent::__construct($entity_manager);
    $this->elementManager = $element_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.manager'),
      $container->get('plugin.manager.yamlform.element')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /* @var $yamlform_submission \Drupal\yamlform\YamlFormSubmissionInterface */
    $yamlform_submission = $this->getEntity();
    $yamlform = $this->getYamlForm();
    $settings = $this->getYamlFormSettings();

    // This submission form is based on the current URL, and hence it depends
    // on the 'url' cache context.
    $form['#cache']['contexts'][] = 'url';

    // Add this YAML form and the YAML form settings to the cache tags.
    $form['#cache']['tags'][] = 'config:yamlform.settings';

    // Add the YAML form as a cacheable dependency.
    \Drupal::service('renderer')->addCacheableDependency($form, $yamlform);

    // Exit if inputs are broken, usually occurs when inputs YAML is edited
    // directly in the export config file.
    if (!$yamlform_submission->getYamlForm()->getInputs()) {
      \Drupal::service('renderer')->addCacheableDependency($form, $this->entity);
      drupal_set_message($this->t('Unable to display this form. Please contact the site administrator.'), 'warning');
      return $form;
    }

    // Display inline confirmation message which is rendered via the controller.
    if ($settings['confirmation_type'] == 'inline' && $this->getRequest()->query->get('yamlform_id') == $yamlform->id()) {
      \Drupal::service('renderer')->addCacheableDependency($form, $this->entity);
      $yamlform_controller = new YamlFormController($this->entityManager->getStorage('yamlform_submission'));
      $form['confirmation'] = $yamlform_controller->confirmation($this->getRequest(), $yamlform);
      return $form;
    }

    // Don't display form if it is closed.
    if ($yamlform_submission->isNew() && $yamlform->isClosed()) {
      \Drupal::service('renderer')->addCacheableDependency($form, $this->entity);
      $form['closed'] = [
        '#markup' => $settings['form_closed_message'],
        '#allowed_tags' => Xss::getAdminTagList(),
      ];
      return $form;
    }

    // Check that submissions will saved to the database or passed to a
    // YamlFormHandler.
    if (!empty($settings['results_disabled']) && !$yamlform->getHandlers()->count()) {
      // Log an error.
      $link = $yamlform->link($this->t('Edit'), 'settings-form');
      $this->logger('yamlform')->error('%form is not saving any submitted data and has been disabled.', ['%form' => $yamlform->label(), 'link' => $link]);

      // Display a warning to admin and disable the form for everyone else.
      if ($this->currentUser()->hasPermission('administer yamlform')) {
        $t_args = [
          ':settings' => $yamlform->toUrl('settings-form')->toString(),
          ':handlers' => $yamlform->toUrl('handlers-form')->toString(),
        ];
        drupal_set_message($this->t('This form is currently not saving any submitted data. Please enable the <a href=":settings">saving of results</a> or add a <a href=":handlers">submission handler</a> to the form.', $t_args), 'error');
      }
      else {
        \Drupal::service('renderer')->addCacheableDependency($form, $this->entity);
        $form['disabled'] = [
          '#markup' => $this->t('This form is unable to process submissions. Please contact the site administrator.'),
          '#allowed_tags' => Xss::getAdminTagList(),
        ];
        return $form;
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /* @var $yamlform_submission \Drupal\yamlform\YamlFormSubmissionInterface */
    $yamlform_submission = $this->getEntity();
    $settings = $this->getYamlFormSettings();

    // Prepend YAML form submission data using the default view without the data.
    if (!$yamlform_submission->isNew()) {
      $view_builder = \Drupal::entityManager()->getViewBuilder('yamlform_submission');
      $yamlform_submission_view = $view_builder->view($yamlform_submission, 'summary');
      $yamlform_submission_build = $view_builder->build($yamlform_submission_view);
      // ISSUE: Adding a nested render array is throwing
      // "A fatal error occurred: The database connection is not serializable."
      // when 'managed_file' is removed via an AJAX request.
      // $form['submission'] = $yamlform_submission_build;
      // WORKAROUND: Render the submission details widget.
      $form['submission'] = ['#markup' => \Drupal::service('renderer')->render($yamlform_submission_build)];
    }

    // Get YAML form inputs.
    $inputs = $yamlform_submission->getYamlForm()->getInputs();

    // Get submission data.
    $data = $yamlform_submission->getData();

    // Prepopulate data using query string parameters.
    if (!empty($settings['form_prepopulate'])) {
      $data = $this->getRequest()->query->all() + $data;
    }

    // Populate YAML form inputs with YAML form submission data.
    $this->populateInputs($inputs, $data);

    // Prepare YAML form inputs.
    $this->prepareInputs($inputs);

    // Handle form with managed file upload but saving of submission is disabled.
    if ($this->getYamlForm()->hasManagedFile() && !empty($settings['results_disabled'])) {
      $link = $this->getYamlForm()->link($this->t('Edit'), 'edit-form');
      $this->logger('yamlform')->notice('To support file uploads the saving of submission must be enabled.. <b>All uploaded load files would be lost</b> Please either uncheck \'Disable saving of submissions\' or remove all the file upload inputs.', $link);
      drupal_set_message($this->t('Unable to display this form. Please contact the site administrator.'), 'warning');
      return $form;
    }

    // Append inputs to the form.
    $form['inputs'] = $inputs;

    // Alter form via YAML form handler.
    $this->getYamlForm()->invokeHandlers('alterForm', $form, $form_state, $yamlform_submission);

    // Add CSS and JS.
    $form['#attached']['library'][] = 'yamlform/yamlform.form';

    // Handle preview.
    if ($form_state->get('yamlform_submission_preview')) {
      // Hide inputs.
      $form['inputs']['#access'] = FALSE;

      // Display preview message.
      drupal_set_message($settings['preview_message'], 'warning');

      // Build preview and display only the data and not the submission information.
      $view_builder = $this->entityManager->getViewBuilder('yamlform_submission');
      $yamlform_submission_view = $view_builder->view($yamlform_submission);
      $preview_elements = $view_builder->build($yamlform_submission_view);
      // Preview must use #tree to generate unique id that don't conflict with
      // the $form['inputs'];
      $form['preview'] = ['#tree' => TRUE] + $preview_elements['data'];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $element = parent::actions($form, $form_state);

    /* @var \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission */
    $yamlform_submission = $this->entity;
    $settings = $this->getYamlFormSettings();
    $preview_mode = $settings['preview'];

    // No delete action on the YAML form submission form.
    unset($element['delete']);

    // Mark the submit action as the primary action, when it appears.
    $element['submit']['#button_type'] = 'primary';

    // Customize the submit button.
    $element['submit']['#value'] = $settings['form_submit_label'];

    if ($preview_mode != DRUPAL_DISABLED) {
      // Only show the save button if YAML form submission previews are optional
      // or if we are already previewing the submission.
      $element['submit']['#access'] = ($yamlform_submission->id() && \Drupal::currentUser()->hasPermission('administer yamlform_submission')) || $preview_mode != DRUPAL_REQUIRED || $form_state->get('yamlform_submission_preview');

      if (!$form_state->get('yamlform_submission_preview')) {
        $element['preview'] = [
          '#type' => 'submit',
          '#value' => $settings['preview_next_button_label'],
          '#submit' => ['::submitForm', '::preview'],
          '#weight' => -1,
        ];
      }
      else {
        $element['back'] = [
          '#type' => 'submit',
          '#value' => $settings['preview_prev_button_label'],
          '#submit' => ['::submitForm', '::back'],
          '#weight' => -1,
        ];
      }
    }

    return $element;
  }

  /**
   * Prepare form inputs.
   *
   * @param array $inputs
   *   An render array representing inputs.
   */
  protected function prepareInputs(array &$inputs) {
    foreach ($inputs as $key => &$element) {
      if (Element::property($key) || !is_array($element)) {
        continue;
      }

      // Remove ignore properties before preparing the element which may need
      // set some ignored properties.
      foreach ($element as $element_property => $element_value) {
        if (isset(self::$ignoredProperties[$element_property])) {
          unset($element[$element_value]);
        }

        // Replace tokens for all properties.
        $element[$element_property] = $this->replaceTokens($element_value);
      }

      // Set #allowed_tags to admin tag list. YAML form builders are
      // to be consider trusted users.
      if (isset($element['#allowed_tags'])) {
        $element['#allowed_tags'] = Xss::getAdminTagList();
      }

      // Handle custom #private access control property.
      if (!empty($element['#private'])) {
        $element['#access'] = YamlFormAccess::checkAdminAccess($this->currentUser());
      }

      // Handle element options via the YamlForm entity.
      if (isset($element['#options'])) {
        $element['#options'] = $this->getYamlForm()->getElementOptions($element);
      }

      // Replace default_value tokens
      // Invoke YamlFormElement::prepare.
      $this->elementManager->invokeMethod('prepare', $element, $this->entity);

      // Initialize default values.
      if (isset($element['#default_value'])) {
        // Invoke YamlFormElement::setDefaultValue.
        $this->elementManager->invokeMethod('setDefaultValue', $element);
      }

      $this->prepareInputs($element);
    }
  }

  /**
   * Populate form inputs.
   *
   * @param array $inputs
   *   An render array representing inputs.
   * @param array $values
   *   An array of values used to populate the inputs.
   */
  protected function populateInputs(array &$inputs, array $values) {
    foreach ($inputs as $key => &$element) {
      // Skip if not a FAPI element.
      if (Element::property($key) || !is_array($element) || !isset($element['#type'])) {
        continue;
      }
      // Populate inputs if value exists.
      if (isset($values[$key])) {
        $element['#default_value'] = $values[$key];
      }
      $this->populateInputs($element, $values);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    // Validate form via YAML form handler.
    $this->getYamlForm()->invokeHandlers('validateForm', $form, $form_state, $this->entity);
  }

  /**
   * Form submission handler for the 'preview' action.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function preview(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getErrors()) {
      $form_state->set('yamlform_submission_preview', TRUE);
      $form_state->setRebuild();
    }
  }

  /**
   * Form submission handler for the 'back' action.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function back(array &$form, FormStateInterface $form_state) {
    $form_state->set('yamlform_submission_preview', FALSE);
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Remove values that are not YAML form inputs.
    $values = YamlFormHelper::cleanupFormStateValues($values, ['uid', 'yamlform_id', 'submit']);

    /* @var $yamlform_submission \Drupal\yamlform\YamlFormSubmissionInterface */
    $yamlform_submission = $this->entity;

    // Serialize the values as YAML.
    $yamlform_submission->setData($values + $yamlform_submission->getData());

    parent::submitForm($form, $form_state);

    // Submit form via YAML form handler.
    $this->getYamlForm()->invokeHandlers('submitForm', $form, $form_state, $yamlform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission */
    $yamlform_submission = $this->getEntity();
    $yamlform = $yamlform_submission->getYamlForm();
    $settings = $yamlform->getSettings();

    if (!$settings['results_disabled']) {
      // Make sure the uri and remote addr are set correctly because
      // AJAX requests via 'managed_file' uploads can cause these values to be
      // reset.
      if ($yamlform_submission->isNew()) {
        $yamlform_submission->set('uri', preg_replace('#^' . base_path() . '#', '/', $this->getRequest()->getRequestUri()));
        $yamlform_submission->set('remote_addr', $this->getRequest()->getClientIp());
      }

      $result = $yamlform_submission->save();
      $link = $yamlform_submission->link($this->t('Edit'), 'edit-form');
      switch ($result) {
        case SAVED_UPDATED:
          $this->logger('yamlform')->notice('Submission updated in %form.', ['%form' => $yamlform->label(), 'link' => $link]);
          break;

        case SAVED_NEW:
          $this->logger('yamlform')->notice('New submission added to %form.', ['%form' => $yamlform->label(), 'link' => $link]);
          break;
      }
    }
    else {
      $result = SAVED_NEW;
    }

    /* Handle confirmation */

    // If new submission and confirmation type is page or URL execute and exit.
    if ($result == SAVED_NEW) {
      $route_options = [];
      if ($query = $this->getRequest()->query->all()) {
        $route_options['query'] = $query;
      }
      if (!$yamlform_submission->isNew()) {
        $route_options['query']['token'] = $yamlform_submission->getToken();
      }

      switch ($settings['confirmation_type']) {
        case 'inline':
          $route_options['query']['yamlform_id'] = $yamlform->id();
          $form_state->setRedirect('<current>', [], $route_options);
          return;

        case 'page':
          $form_state->setRedirect('entity.yamlform.confirmation', ['yamlform' => $yamlform->id()], $route_options);
          return;

        case 'url':
          if ($confirmation_url = \Drupal::pathValidator()->getUrlIfValid(trim($settings['confirmation_url'])) ?: NULL) {
            if ($settings['confirmation_message']) {
              drupal_set_message($settings['confirmation_message']);
            }
            $form_state->setRedirectUrl($confirmation_url);
            return;
          }
      }
    }

    // Default to displaying a confirmation message on this page.
    switch ($result) {
      case SAVED_UPDATED:
        drupal_set_message($this->t('Submission updated in %form.', ['%form' => $yamlform->label()]));
        $form_state->setRedirect('entity.yamlform_submission.canonical', ['yamlform_submission' => $yamlform_submission->id()]);
        break;

      case SAVED_NEW:
        if ($settings['confirmation_message']) {
          drupal_set_message($settings['confirmation_message']);
        }
        else {
          drupal_set_message($this->t('New submission added to %form.', ['%form' => $yamlform->label()]));
        }

        $route_parameters = ['yamlform' => $yamlform->id()];
        $route_options = [];
        if ($query = $this->getRequest()->query->all()) {
          $route_options['query'] = $query;
        }
        $form_state->setRedirect('entity.yamlform.canonical', $route_parameters, $route_options);
        break;
    }
  }

  /**
   * Get the YAML form submission's YAML form.
   *
   * @return \Drupal\yamlform\Entity\YamlForm
   *   A YAML form.
   */
  protected function getYamlForm() {
    /** @var \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission */
    $yamlform_submission = $this->getEntity();
    return $yamlform_submission->getYamlForm();
  }

  /**
   * Get the YAML form submission's YAML form settings.
   *
   * @return array
   *   A specific settings or an associative array of settings.
   */
  protected function getYamlFormSettings() {
    if (empty($this->settings)) {
      // Get YAML form settings with  default values.
      $this->settings = $this->getYamlForm()->getSettings();
      $default_settings = $this->config('yamlform.settings')->get('settings');
      foreach ($default_settings as $key => $value) {
        $key = str_replace('default_', '', $key);
        if (empty($this->settings[$key])) {
          $this->settings[$key] = $value;
        }
      }
    }

    return $this->settings;
  }

  /**
   * Replace tokens in text.
   *
   * @param string $text
   *   A string of text that main contain tokens.
   *
   * @return string
   *  Text will tokens replaced.
   */
  protected function replaceTokens($text) {
    // Most strings won't contain tokens so lets check and return ASAP.
    if (!is_string($text) || strpos($text, '[') === FALSE) {
      return $text;
    }
    $token_data = [
      'yamlform' => $this->getYamlForm(),
      'yamlform_submission' => $this->entity,
    ];
    return \Drupal::token()->replace($text, $token_data);
  }

}
