<?php

/**
 * @file
 * Contains \Drupal\yamlform\Entity\YamlFormSubmissionListBuilder.
 */

namespace Drupal\yamlform;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;

/**
 * Provides a list controller for yamlform submission entity.
 *
 * @ingroup yamlform
 */
class YamlFormSubmissionListBuilder extends EntityListBuilder {

  /**
   * The YAML form.
   *
   * @var \Drupal\yamlform\Entity\YamlForm
   */
  protected $yamlform;

  /**
   * YAML form view builder.
   *
   * @var \Drupal\Core\Entity\EntityViewBuilderInterface
   */
  protected $viewBuilder;
  /**
   * The YAML form input columns.
   *
   * @var array
   */
  protected $columns = [];

  /**
   * The YAML form results filter search keys.
   *
   * @var string
   */
  protected $keys;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage) {
    parent::__construct($entity_type, $storage);
    $this->keys = \Drupal::request()->query->get('search');
    $this->yamlform = \Drupal::routeMatch()->getParameter('yamlform');
    if ($this->yamlform && \Drupal::routeMatch()->getRouteName() == 'entity.yamlform.results_table') {
      $this->columns = $this->yamlform->getInputColumns();
    }

    $this->viewBuilder = \Drupal::entityManager()->getViewBuilder('yamlform_submission');
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    if ($this->yamlform) {
      $build['#title'] = $this->yamlform->label();
    }

    // Add the filter.
    $build['filter_form'] = \Drupal::formBuilder()->getForm('\Drupal\yamlform\Form\YamlFormFilterForm', $this->t('submissions'), $this->keys);

    // Display info.
    if ($total = $this->getTotal()) {
      $t_args = [
        '@total' => $total,
        '@results' => $this->formatPlural($total, $this->t('submission'), $this->t('submissions')),
      ];
      $build['info'] = [
        '#markup' => $this->t('@total @results', $t_args),
        '#prefix' => '<div>',
        '#suffix' => '</div>',
      ];
    }

    $build += parent::render();

    $build['table']['#attributes']['class'][] = 'yamlform-results';

    $build['#attached']['library'][] = 'yamlform/yamlform';

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['sid'] = $this->t('#');
    $header['created'] = $this->t('Submitted');
    $header['entity'] = $this->t('Submitted to');
    $header['user'] = $this->t('User');
    $header['remote_addr'] = $this->t('IP address');

    if (empty($this->yamlform)) {
      $header['yamlform'] = $this->t('Form');
    }

    if ($this->columns) {
      foreach ($this->columns as $key => $column) {
        $header['input_' . $key] = $column['title'];
      }
    }

    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /* @var $entity \Drupal\yamlform\YamlFormSubmissionInterface */
    $row['sid'] = $entity->toLink($entity->id());
    $row['created'] = format_date($entity->created->value);
    $row['entity'] = ($source_entity = $entity->getSourceEntity()) ? $source_entity->toLink() : '';
    $row['user'] = $entity->getOwner()->getAccountName();
    $row['remote_addr'] = $entity->remote_addr->value;

    if (empty($this->yamlform)) {
      $row['yamlform'] = $entity->getYamlForm()->toLink();
    }

    if ($this->columns) {
      $data = $entity->getData();
      foreach ($this->columns as $key => $column) {
        $row['input_' . $key] = $this->viewBuilder->formatElementValueAsHtml($column['element'], $data[$key]);
      }
    }

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = parent::getDefaultOperations($entity);

    // Add destination to edit and delete operations.
    foreach ($operations as &$operation) {
      $operation['url']->setOptions(['query' => drupal_get_destination()]);
    }

    // Add view to operations.
    $operations['view'] = [
      'title' => $this->t('View'),
      'weight' => 20,
      'url' => Url::fromRoute('entity.yamlform_submission.canonical', ['yamlform_submission' => $entity->id()]),
    ];

    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    return $this->getQuery()
      ->sort($this->entityType->getKey('id'), 'DESC')
      ->pager($this->limit)
      ->execute();
  }

  /**
   * Get the total number of submissions.
   *
   * @return int
   *   The total number of submissions.
   */
  protected function getTotal() {
    return $this->getQuery()
      ->count()
      ->execute();
  }

  /**
   * Get the base entity query filtered by YAML form and search.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   An entity query.
   */
  protected function getQuery() {
    $query = $this->getStorage()->getQuery();

    // Limit submission to the current YAML form.
    if ($this->yamlform) {
      $query->condition('yamlform_id', $this->yamlform->id());
    }

    // Filter submissions
    if ($this->keys) {
      $query->condition('data', '%' . $this->keys . '%', 'LIKE');
    }

    return $query;
  }

}
