<?php

/**
 * @file
 * Contains \Drupal\yamlform\YamlFormSubmissionViewBuilder.
 */

namespace Drupal\yamlform;

use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Utility\Token;
use Drupal\yamlform\Utility\YamlFormOptionsHelper;
use Drupal\yamlform\Utility\YamlFormTidy;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render controller for YAML form submissions.
 */
class YamlFormSubmissionViewBuilder extends EntityViewBuilder {

  /**
   * Array of element properties that should be copied to #type='item' properties.
   *
   * @var array
   */
  protected $itemProperties = [
    '#title' => '#title',
  ];

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The token handler.
   *
   * @var \Drupal\Core\Utility\Token $token
   */
  protected $token;

  /**
   * The YAML form handler manager service.
   *
   * @var \Drupal\yamlform\YamlFormHandlerManager
   */
  protected $yamlFormHandlerManager;

  /**
   * The YAML form element manager service.
   *
   * @var \Drupal\yamlform\YamlFormElementManager
   */
  protected $yamlFormElementManager;

  /**
   * Constructs a new YamlFormSubmissionViewBuilder.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Utility\Token $token
   *   The token handler.
   * @param \Drupal\yamlform\YamlFormHandlerManager $yamlform_handler_manager
   *   The YAML form handler manager service.
   * @param \Drupal\yamlform\YamlFormElementManager $yamlform_element_manager
   *   The YAML form element manager service.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityManagerInterface $entity_manager, LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory, Token $token, YamlFormHandlerManager $yamlform_handler_manager, YamlFormElementManager $yamlform_element_manager) {
    parent::__construct($entity_type, $entity_manager, $language_manager);
    $this->configFactory = $config_factory;
    $this->token = $token;
    $this->yamlFormHandlerManager = $yamlform_handler_manager;
    $this->yamlFormElementManager = $yamlform_element_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.manager'),
      $container->get('language_manager'),
      $container->get('config.factory'),
      $container->get('token'),
      $container->get('plugin.manager.yamlform.handler'),
      $container->get('plugin.manager.yamlform.element')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildComponents(array &$build, array $entities, array $displays, $view_mode) {
    switch ($view_mode) {
      case 'yaml':
        foreach ($entities as $id => $entity) {
          $build[$id]['yaml'] = $this->buildYaml($entity);
        }
        break;

      case 'text':
        foreach ($entities as $id => $entity) {
          $build[$id]['text'] = $this->buildText($entity);
        }
        break;

      default:
        parent::buildComponents($build, $entities, $displays, $view_mode);
        foreach ($entities as $id => $entity) {
          // Link the entity id to the source entity.
          unset($build[$id]['entity_type']);
          if ($source_entity = $entity->getSourceEntity()) {
            $build[$id]['entity_id']['#title'] = 'Submitted to';
            $build[$id]['entity_id'][0] = [
              '#type' => 'link',
              '#title' => $source_entity->label(),
              '#url' => $source_entity->toUrl(),
            ];
          }

          // Link the source uri.
          if ($url = $entity->getSourceUrl()) {
            $build[$id]['uri'][0] = [
              '#type' => 'link',
              '#title' => $entity->uri->value,
              '#url' => $url,
            ];
          }

          // Convert YAML form submission to form items.
          unset($build[$id]['data']);
          if ($view_mode != 'summary') {
            $yamlform = $entity->getYamlForm();
            $data = $entity->getData();
            $inputs = $yamlform->getInputs();
            $build[$id]['data'] = $this->buildInputsAsItems($inputs, $data);
          }
        }
        break;

    }
  }

  /**
   * {@inheritdoc}
   */
  protected function alterBuild(array &$build, EntityInterface $entity, EntityViewDisplayInterface $display, $view_mode) {
    /* @var $entity \Drupal\yamlform\YamlFormSubmissionInterface */
    switch ($view_mode) {
      case 'yaml':
      case 'text':
        return $build;

      default:
        $build['#attached']['library'][] = 'yamlform/yamlform';

        // Wrap submission information in a details widget.
        $build['submission'] = [
          '#type' => 'details',
          '#title' => $this->t('Submission information'),
          '#open' => TRUE,
          '#weight' => -20,
        ];
        foreach ($build as $key => $element) {
          if (!Element::property($key) && !in_array($key, ['data', 'submission'])) {
            $build['submission'][$key] = $element;
            unset($build[$key]);
          }
        }
        return $build;
    }
  }

  /**
   * Build a YAML representation for a YAML form submission.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   A render array displaying a YAML form submission as YAML.
   */
  protected function buildYaml(YamlFormSubmissionInterface $yamlform_submission) {
    $build = [];
    foreach ($yamlform_submission->getFieldDefinitions() as $key => $definition) {
      $item = $yamlform_submission->get($key);
      $value = (isset($item->target_id)) ? $item->target_id : $item->value;
      if ($key == 'data') {
        $value = str_replace("\n", "\n  ", YamlFormTidy::tidy($value));
        $build[$key] = [
          '#markup' => "$key: |\n  $value",
          '#suffix' => "\n",
        ];
      }
      else {
        if (in_array($key, ['remote_addr'])) {
          $value = "'$value'";
        }
        $build[$key] = [
          '#markup' => "$key: $value",
          '#suffix' => "\n",
        ];
      }
    }
    return $build;
  }

  /**
   * Build a plain text representation for a YAML form submission.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   A render array displaying a YAML form submission as plain text.
   */
  protected function buildText(YamlFormSubmissionInterface $yamlform_submission) {
    $text = $this->configFactory->get('yamlform.settings')->get('display.submission_text');
    $token_data = [
      'yamlform' => $yamlform_submission->getYamlForm(),
      'yamlform-submission' => $yamlform_submission,
    ];
    return [
      '#markup' => $this->token->replace($text, $token_data),
    ];
  }

  /**
   * Build element display items from inputs and submitted data.
   *
   * @param array $elements
   *   A render array of form elements.
   * @param array $data
   *   Submission data.
   * @param array $options
   *   - excluded_inputs: An array of inputs to be excluded.
   *
   * @return array
   *   A render array displaying the submitted values.
   */
  protected function buildInputsAsItems(array $elements, array $data, array $options = []) {
    $build = [];

    foreach ($elements as $key => $element) {
      if (!is_array($element) || Element::property($key) || !$this->isVisibleElement($element) || isset($options['excluded_inputs'][$key])) {
        continue;
      }

      $element += [
        '#type' => NULL,
        '#title' => NULL,
      ];

      if (empty($element['#type']) || in_array($element['#type'], ['item', 'label'])) {
        $build += $this->buildInputsAsItems($element, $data);
        continue;
      }

      $is_container = in_array($element['#type'], ['container']);
      $is_group = in_array($element['#type'], ['details', 'fieldset']);
      if ($is_group || $is_container) {
        $children = $this->buildInputsAsItems($element, $data);
        if (!empty($children)) {
          if ($is_group) {
            $build[$key] = array_intersect_key($element, $this->itemProperties);
            $build[$key]['#type'] = 'details';
            $build[$key]['#open'] = TRUE;
            $build[$key] += $children;
          }
          else {
            $build[$key] = $children;
          }
        }
      }
      else {
        $has_value = (isset($data[$key]) && $data[$key] !== '');
        if ($has_value) {
          $build[$key] = array_intersect_key($element, $this->itemProperties);
          $build[$key]['#type'] = 'item';
          $build[$key]['#name'] = $key;
          $build[$key]['#markup'] = $this->formatElementValueAsHtml($element, $data[$key]);
        }
      }
    }
    return $build;
  }

  /**
   * Build a very simple plain-text render of a submissions values.
   *
   * @param array $elements
   *   The elements/inputs.
   * @param array $data
   *   A YAML form submission data.
   * @param array $options
   *   - excluded_inputs: An array of inputs to be excluded.
   *
   * @return array
   *   A render array containing just plain next.
   */
  public function buildInputAsText(array $elements, array $data, array $options = []) {
    $build = [];

    foreach ($elements as $key => $element) {
      if (!is_array($element) || Element::property($key) || !$this->isVisibleElement($element) || isset($options['excluded_inputs'][$key])) {
        continue;
      }

      $element += [
        '#type' => NULL,
        '#title' => NULL,
      ];

      if (empty($element['#type']) || in_array($element['#type'], ['item', 'label'])) {
        $build += $this->buildInputAsText($element, $data, $options);
        continue;
      }

      $is_container = in_array($element['#type'], ['container']);
      $is_group = in_array($element['#type'], ['details', 'fieldset']);
      if ($is_group || $is_container) {
        $children = $this->buildInputAsText($element, $data, $options);
        if (!empty($children)) {
          if ($is_group) {
            $build[$key] = ['#prefix' => "\n", '#suffix' => "\n"];
            if ($element['#title']) {
              $build[$key]['title'] = [
                '#markup' => strtoupper($element['#title']),
                '#suffix' => "\n" . str_repeat('-', strlen($element['#title'])) . "\n",
              ];
            }
            if ($children) {
              $build[$key]['items'] = $children;
            }
          }
          else {
            $build[$key] = $children;
          }
        }
      }
      else {
        $has_value = (isset($data[$key]) && $data[$key] !== '');
        $title = $element['#title'];
        $value = $this->formatElementValueAsText($element, $data[$key]);
        if ($has_value) {
          if ($title) {
            $build[$key] = [
              '#markup' => $title . (in_array($element['#type'], ['textarea', 'text_format']) ? ":\n" : ": ") . $value,
              '#suffix' => "\n",
            ];
          }
          else {
            $build[$key] = [
              '#markup' => $value,
              '#suffix' => "\n",
            ];
          }
        }
      }
    }

    return $build;
  }

  /**
   * Format input element value as HTML.
   *
   * @param array $element
   *   Form element.
   * @param string|int $value
   *   A value.
   *
   * @return string
   *   Value as HTML.
   */
  public function formatElementValueAsHtml(array $element, $value) {
    // Return empty value.
    if ($value === '' || $value === NULL) {
      return '';
    }

    // Invoke YamlFormElement::formatHtml.
    $element_value = $this->yamlFormElementManager->invokeMethod('formatHtml', $element, $value);
    if ($element_value !== NULL) {
      return $element_value;
    }

    // Default to using plain text formatting.
    if (isset($element['#type'])) {
      return $this->formatElementValueAsText($element, $value);
    }

    return $value;

  }

  /**
   * Format input element value as plain text.
   *
   * @param array $element
   *   Form element.
   * @param string|int $value
   *   A value.
   *
   * @return string
   *   Value as plain text.
   */
  public function formatElementValueAsText(array $element, $value) {
    // Return empty value.
    if ($value === '' || $value === NULL) {
      return '';
    }

    // Convert option value(s) to a comma delimited string.
    if (isset($element['#options']) && is_array($element['#options'])) {
      $flattened_options = YamlFormOptionsHelper::flattenOptions($element['#options']);
      if (is_array($value)) {
        return implode(', ', YamlFormOptionsHelper::getOptionsText($value, $flattened_options));
      }
      else {
        return YamlFormOptionsHelper::getOptionText($value, $flattened_options);
      }
    }

    // Invoke YamlFormElement::formatText.
    $element_value = $this->yamlFormElementManager->invokeMethod('formatText', $element, $value);
    if ($element_value !== NULL) {
      return $element_value;
    }

    // Format value based on the element type using default settings.
    if (isset($element['#type'])) {
      // Apply #field prefix and #field_suffix to value.
      if (isset($element['#field_prefix'])) {
        $value = $element['#field_prefix'] . $value;
      }
      if (isset($element['#field_suffix'])) {
        $value .= $element['#field_suffix'];
      }
    }

    return $value;
  }

  /**
   * Determines if an element is visible.
   *
   * Copied from: \Drupal\Core\Render\Element::isVisibleElement
   * but does not hide hidden or value elements.
   *
   * @param array $element
   *   The element to check for visibility.
   *
   * @return bool
   *   TRUE if the element is visible, otherwise FALSE.
   */
  public function isVisibleElement($element) {
    return (isset($element['#type']))
      && (!isset($element['#access']) || (($element['#access'] instanceof AccessResultInterface && $element['#access']->isAllowed()) || ($element['#access'] === TRUE)));
  }

}
