<?php

/**
 * @file
 * Builds placeholder replacement tokens for YAML forms and submission.
 */

use Drupal\Core\Render\BubbleableMetadata;
use Drupal\yamlform\Entity\YamlForm;
use Drupal\Core\Datetime\Entity\DateFormat;

/**
 * Implements hook_token_info().
 */
function yamlform_token_info() {
  $types['yamlform-submission'] = [
    'name' => t("YAML form submissions"),
    'description' => t("Tokens related to YAML form submissions."),
    'needs-data' => 'yamlform-submission',
  ];
  $types['yamlform'] = [
    'name' => t("YAML forms"),
    'description' => t("Tokens related to YAML forms."),
    'needs-data' => 'yamlform',
  ];

  // YAML form submission related variables.
  $yamlform_submission['sid'] = [
    'name' => t("YAML form submission ID"),
    'description' => t("The ID of the YAML form submission ."),
  ];
  $yamlform_submission['uuid'] = [
    'name' => t('UUID'),
    'description' => t('The UUID of the YAML form submission.'),
  ];
  $yamlform_submission['token'] = [
    'name' => t('Token'),
    'description' => t('A secure token used to look up a submission.'),
  ];
  $yamlform_submission['ip-address'] = [
    'name' => t('IP address'),
    'description' => t('The IP address that was used when submitting the YAML form submission.'),
  ];
  $yamlform_submission['source-url'] = [
    'name' => t('Source URL'),
    'description' => t("The URL the user submitted the YAML form submission."),
  ];
  $yamlform_submission['url'] = [
    'name' => t('URL'),
    'description' => t('The URL of the YAML form submission. Replace the "?" with the link template. Defaults to "canonical" which displays the submission\'s data.'),
    'dynamic' => TRUE,
  ];
  $yamlform_submission['user'] = [
    'name' => t('Submitter'),
    'description' => t('The user that submitted the YAML form submission.'),
    'type' => 'user',
  ];
  $yamlform_submission['values'] = [
    'name' => t('YAML form submission values'),
    'description' => t('YAML form tokens from submitted data. Replace the "?" with the "input name".'),
    'dynamic' => TRUE,
  ];

  // YAML form related variables.
  $yamlform['id'] = [
    'name' => t("YAML form ID"),
    'description' => t("The ID of the YAML form."),
  ];
  $yamlform['title'] = [
    'name' => t("title"),
    'description' => t("The title of the YAML form."),
  ];

  $yamlform['description'] = [
    'name' => t("Description"),
    'description' => t("The admistrative description of the YAML form."),
  ];
  $yamlform['url'] = [
    'name' => t('URL'),
    'description' => t('The URL of the YAML form. Replace the "?" with the link template. Defaults to "canonical" which displays the submission form.'),
    'dynamic' => TRUE,
  ];

  // Chained tokens for YAML form submissions.
  $yamlform_submission['created'] = [
    'name' => t("Date created"),
    'description' => t("The date the comment was posted."),
    'type' => 'date',
  ];
  $yamlform_submission['changed'] = [
    'name' => t("Date changed"),
    'description' => t("The date the comment was most recently updated."),
    'type' => 'date',
  ];
  $yamlform_submission['yamlform'] = [
    'name' => t("YAML form"),
    'description' => t("The YAML form that the YAML form submission belongs to."),
    'type' => 'yamlform',
  ];
  $yamlform_submission['source-entity'] = [
    'name' => t("Source entity"),
    'description' => t("The entity that the YAML form submission was submitted from."),
    'type' => 'entity',
  ];

  return [
    'types' => $types,
    'tokens' => [
      'yamlform-submission' => $yamlform_submission,
      'yamlform' => $yamlform,
    ],
  ];
}

/**
 * Implements hook_tokens().
 */
function yamlform_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {
  $token_service = \Drupal::token();

  $url_options = ['absolute' => TRUE];
  if (isset($options['langcode'])) {
    $url_options['language'] = \Drupal::languageManager()->getLanguage($options['langcode']);
    $langcode = $options['langcode'];
  }
  else {
    $langcode = NULL;
  }

  $replacements = [];

  if ($type == 'yamlform-submission' && !empty($data['yamlform-submission'])) {

    /** @var \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission */
    $yamlform_submission = $data['yamlform-submission'];

    /** @var \Drupal\yamlform\YamlFormSubmissionViewBuilder $view_builder */
    $view_builder = \Drupal::entityManager()->getViewBuilder('yamlform_submission');

    $form_inputs = $yamlform_submission->getYamlForm()->getFlattenedInputs();
    $submission_data = $yamlform_submission->getData();

    foreach ($tokens as $name => $original) {
      if (strpos($name, 'values:') === 0) {
        $key = str_replace('values:', '', $name);
        if (isset($submission_data[$key]) && isset($form_inputs[$key])) {
          $replacements[$original] = $view_builder->formatElementValueAsText($form_inputs[$key], $submission_data[$key]);
        }
      }
      elseif (strpos($name, 'url:') === 0) {
        $key = str_replace('url:', '', $name);
        if ($yamlform_submission->id() && $yamlform_submission->hasLinkTemplate($key)) {
          $replacements[$original] = $yamlform_submission->toUrl($key)->toString();
        }
      }
      else {

        switch ($name) {
          case 'sid':
            $replacements[$original] = $yamlform_submission->id();
            break;

          case 'uuid':
            $replacements[$original] = $yamlform_submission->uuid->value;
            break;

          case 'ip-address':
            $replacements[$original] = $yamlform_submission->remote_addr->value;
            break;

          case 'token':
            $replacements[$original] = $yamlform_submission->getToken();
            break;

          case 'user':
            /** @var \Drupal\Core\Session\AccountInterface $account */
            $account = $yamlform_submission->uid->entity;
            $replacements[$original] = $account->getDisplayName();
            break;

          case 'created':
          case 'changed':
            $date_format = DateFormat::load('medium');
            $bubbleable_metadata->addCacheableDependency($date_format);
            $replacements[$original] = format_date($yamlform_submission->{$name}->value, 'medium', '', NULL, $langcode);
            break;

          case 'url':
            if ($yamlform_submission->id()) {
              $replacements[$original] = $yamlform_submission->toUrl('canonical', ['absolute' => TRUE])->toString();
            }
            break;

          case 'source-url':
            $replacements[$original] = $yamlform_submission->getSourceUrl()->toString();
            break;

          case 'source-entity':
            $replacements[$original] = $yamlform_submission->getSourceEntity()->label();
            break;

          case 'values':
            $submission_options = isset($data['yamlform-submission-options']) ? $data['yamlform-submission-options'] : [];
            $build = $view_builder->buildInputAsText($yamlform_submission->getYamlForm()->getInputs(), $submission_data, $submission_options);
            $replacements[$original] = trim((string) drupal_render($build));
            break;

          case 'yamlform':
            $yamlform = $yamlform_submission->getYamlForm();
            $bubbleable_metadata->addCacheableDependency($yamlform);
            $replacements[$original] = $yamlform->label();
            break;

        }
      }
    }

    // Chained token relationships.
    if ($yamlform_tokens = $token_service->findWithPrefix($tokens, 'yamlform')) {
      $yamlform = YamlForm::load($yamlform_submission->bundle());
      $replacements += $token_service->generate('yamlform', $yamlform_tokens, ['yamlform' => $yamlform], $options, $bubbleable_metadata);
    }
    if ($entity_tokens = $token_service->findwithPrefix($tokens, 'source-entity')) {
      $entity = $yamlform_submission->getSourceEntity();
      $replacements += $token_service->generate($entity->getEntityTypeId(), $entity_tokens, [$entity->getEntityTypeId() => $entity], $options, $bubbleable_metadata);
    }

  }
  elseif ($type == 'yamlform' && !empty($data['yamlform'])) {

    $yamlform = $data['yamlform'];
    foreach ($tokens as $name => $original) {
      if (strpos($name, 'url:') === 0) {
        $key = str_replace('url:', '', $name);
        if ($yamlform->hasLinkTemplate($key)) {
          $replacements[$original] = $yamlform->toUrl($key)->toString();
        }
      }
      else {
        switch ($name) {
          case 'id':
            $replacements[$original] = $yamlform->id();
            break;

          case 'title':
            $replacements[$original] = $yamlform->label();
            break;

          case 'description':
            $replacements[$original] = $yamlform->getDescription();
            break;

          case 'url':
            $replacements[$original] = $yamlform->toUrl('canonical', ['absolute' => TRUE])->toString();
            break;

        }
      }
    }
  }

  return $replacements;
}
